package vip.xiaonuo.erp.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.sun.xml.bind.v2.runtime.output.SAXOutput;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;
import vip.xiaonuo.erp.entity.ErpBdOrg;
import vip.xiaonuo.erp.entity.ErpBdRawMaterial;
import vip.xiaonuo.erp.entity.kucun.*;
import vip.xiaonuo.erp.mapper.ErpBdOrgMapper;
import vip.xiaonuo.erp.mapper.ErpBdRawMaterialMapper;
import vip.xiaonuo.erp.service.IErpKuCunService;
import vip.xiaonuo.warehouse.entity.*;
import vip.xiaonuo.warehouse.entity.vo.FSAFESTOCK;
import vip.xiaonuo.warehouse.mapper.*;

import java.util.Date;
import java.util.List;


@Service
public class ErpKuCunServiceImpl implements IErpKuCunService {


    @Autowired
    private RestTemplate restTemplate;
    @Autowired
    private ErpBdOrgMapper erpBdOrgMapper;
    @Autowired
    private WarehouseIncomStockMapper incomStockMapper;
    @Autowired
    private WarehouseOutStockMapper outStockMapper;
    @Autowired
    private WarehouseStockMapper stockMapper;
    @Autowired
    private WarehouseMaterialStockMapper materialStockMapper;
    @Autowired
    private WarehouseStockDetailMapper stockDetailMapper;
    @Autowired
    private WarehouseMaterialOutStockMapper materialOutStockMapper;
    @Autowired
    private WarehouseMaterialInStockMapper materialInStockMapper;
    @Autowired
    private WarehouseStockSecondMapper stockSecondMapper;
    @Autowired
    private ErpBdRawMaterialMapper erpBdRawMaterialMapper;

    @Autowired
    private WarehouseWarningMapper warehouseWarningMapper;


    //获取安全库存
    @Override
    public void getFSAFESTOCK() {
        Date date = DateUtil.beginOfDay(new Date());  // 获取当前日期
        // 构建URL，使用格式化的日期
        ResponseEntity<List<FSAFESTOCK>> response = restTemplate.exchange(
                "http://127.0.0.1:9200/erp-kucun/getFSAFESTOCK?date=",
                HttpMethod.GET,
                null,
                new ParameterizedTypeReference<List<FSAFESTOCK>>() {
                }
        );
        List<FSAFESTOCK> forObjects = response.getBody();
        System.out.println(forObjects);
        for (FSAFESTOCK forObject : forObjects) {
            ErpBdOrg forgid = erpBdOrgMapper.selectOne(new QueryWrapper<ErpBdOrg>().eq("forgid", forObject.getFUSEORGID()));
            if (ObjectUtil.isEmpty(forgid))
                continue;
            ErpBdRawMaterial erpBdRawMaterial = erpBdRawMaterialMapper.selectOne(new QueryWrapper<ErpBdRawMaterial>().eq("org_id", forgid.getOrgId()).eq("erp_fnumber", forObject.getFNUMBER()));
            if (ObjectUtil.isNotEmpty(erpBdRawMaterial)){
                erpBdRawMaterial.setSafeStock((int)Math.floor(forObject.getFSAFESTOCK()));
                erpBdRawMaterialMapper.updateById(erpBdRawMaterial);
            }
        }

    }

    @Override
    public void getYuanCaiLiaoCunLiang(Date date) {
        // 构建URL，使用格式化的日期
        ResponseEntity<List<YuanCaiLiaoKuCun>> response = restTemplate.exchange(
                "http://127.0.0.1:9200/erp-kucun/getYuanCaiLiaoCunLiang?date="+date,
                HttpMethod.GET,
                null,
                new ParameterizedTypeReference<List<YuanCaiLiaoKuCun>>() {
                }
        );
        List<YuanCaiLiaoKuCun> forObjects = response.getBody();
        System.out.println(forObjects);
        for (YuanCaiLiaoKuCun forObject : forObjects) {
            ErpBdOrg forgid = erpBdOrgMapper.selectOne(new QueryWrapper<ErpBdOrg>().eq("forgid", forObject.getFSTOCKORGID()));
            if (ObjectUtil.isEmpty(forgid))
                continue;
            WarehouseStock warehouseStock = new WarehouseStock();
            WarehouseStockDetail warehouseStockDetail = new WarehouseStockDetail();
            //原材料
            if (forObject.getFnumber().startsWith("10") && !forObject.getFnumber().equals("10020002")){
                warehouseStock.setMaterial(String.format("%.2f",forObject.getFBASEQTY()));
                warehouseStockDetail.setMaterialCode(0);
            }
            //成品、半成品
            if (forObject.getFnumber().startsWith("01") ){
                warehouseStock.setFinishProduce(String.format("%.2f",forObject.getFBASEQTY()));
                warehouseStockDetail.setMaterialCode(1);
            }
            //低值易耗品
            if (forObject.getFnumber().startsWith("15")){
                warehouseStock.setLowValue(String.format("%.2f",forObject.getFBASEQTY()));
                warehouseStockDetail.setMaterialCode(3);
            }
            //包材
            if (forObject.getFnumber().startsWith("11")){
                warehouseStock.setPackageMaterial(String.format("%.2f",forObject.getFBASEQTY()));
                warehouseStockDetail.setMaterialCode(4);
            }
            //羊毛脂
            if ("10020002".equals(forObject.getFnumber())){
                warehouseStock.setLanolin(String.format("%.2f",forObject.getFBASEQTY()));
                warehouseStockDetail.setMaterialCode(5);
            }
            //鱼油
            if ("10020071".equals(forObject.getFnumber())){
                warehouseStock.setFishOil(String.format("%.2f",forObject.getFBASEQTY()));
                warehouseStockDetail.setMaterialCode(6);
            }
            warehouseStock.setCreateTime(forObject.getFUPDATETIME());
            warehouseStock.setOrgId(forgid.getOrgId());
            stockMapper.insert(warehouseStock);

            //库存
            WarehouseMaterialStock warehouseMaterialStock = new WarehouseMaterialStock();
            warehouseMaterialStock.setCreateTime(forObject.getFUPDATETIME());
            warehouseMaterialStock.setMaterial(forObject.getWuLiaoName());
            warehouseMaterialStock.setMaterialType(getMaterialType(forObject.getFNAME()));
            warehouseMaterialStock.setMaterialStock(String.format("%.2f",forObject.getFBASEQTY()));
            warehouseMaterialStock.setOrgId(forgid.getOrgId());
            warehouseMaterialStock.setOrgName(forgid.getFname());
            warehouseMaterialStock.setFnumber(forObject.getFnumber());
            materialStockMapper.insert(warehouseMaterialStock);


            warehouseStockDetail.setOrgId(forgid.getOrgId());
//            warehouseStockDetail.setMaterialCode(forObject.getFMATERIALID());
            warehouseStockDetail.setMaterialName(forObject.getWuLiaoName());
            warehouseStockDetail.setMaterialNumber(String.format("%.2f",forObject.getFBASEQTY()));
            warehouseStockDetail.setStockName(forObject.getFNAME());
            warehouseStockDetail.setCreateTime(forObject.getFUPDATETIME());
            warehouseStockDetail.setFnumber(forObject.getFnumber());
            stockDetailMapper.insert(warehouseStockDetail);

            WarehouseStockSecond warehouseStockSecond = new WarehouseStockSecond();
            if (("原料库".equals(forObject.getFNAME()) || "高科原料仓库".equals(forObject.getFNAME())) && !forObject.getFnumber().equals("10020002")){
                warehouseStockSecond.setMaterialType(0);
            }
            if ("成品库".equals(forObject.getFNAME()) || "成品库CW".equals(forObject.getFNAME())){
                warehouseStockSecond.setMaterialType(1);
            }
            if ("五金库".equals(forObject.getFNAME())){
                warehouseStockSecond.setMaterialType(3);
            }
            if ("包材库".equals(forObject.getFNAME())){
                warehouseStockSecond.setMaterialType(4);
            }
            if (forObject.getFnumber().equals("10020002")){
                warehouseStockSecond.setMaterialType(5);
            }
            if ("10020071".equals(forObject.getFnumber())){
                warehouseStockSecond.setMaterialType(6);
            }
            warehouseStockSecond.setOrgId(forgid.getOrgId());
            warehouseStockSecond.setStock(String.format("%.2f",forObject.getFBASEQTY()));
            warehouseStockSecond.setCreateTime(forObject.getFUPDATETIME());
            stockSecondMapper.insert(warehouseStockSecond);
            //安全预警
            ErpBdRawMaterial erpBdRawMaterial = erpBdRawMaterialMapper.selectOne(new QueryWrapper<ErpBdRawMaterial>().eq("materal_id", forObject.getFMATERIALID()));

            if (ObjectUtil.isNotEmpty(forObject) && ObjectUtil.isNotEmpty(erpBdRawMaterial)
                    && ObjectUtil.isNotEmpty(forObject.getFBASEQTY()) && ObjectUtil.isNotEmpty(erpBdRawMaterial.getSafeStock())){
                if (forObject.getFBASEQTY() < erpBdRawMaterial.getSafeStock()){
                    WarehouseWarning warehouseWarning = new WarehouseWarning();
                    warehouseWarning.setCangku(forObject.getFNAME());
                    warehouseWarning.setCreateTime(forObject.getFUPDATETIME());
                    warehouseWarning.setFnumber(forObject.getFnumber());
                    warehouseWarning.setMaterialid(forObject.getFMATERIALID());
                    warehouseWarning.setOrgId(forgid.getOrgId());
                    warehouseWarning.setWarehouseName(forObject.getWuLiaoName());
                    warehouseWarning.setWarehouseQty(String.format("%.2f",forObject.getFBASEQTY()));
                    warehouseWarning.setWarehouseSafeQty(erpBdRawMaterial.getSafeStock());
                    warehouseWarningMapper.insert(warehouseWarning);
                }
            }

//
        }

    }



    @Override
    public void getChengPinDangRi(Date date) {
        //成品半成品
        ResponseEntity<List<ChengPinRuKu>> response1 = restTemplate.exchange(
                "http://127.0.0.1:9200/erp-kucun/getChengPinDangRi?date="+date,
                HttpMethod.GET,
                null,
                new ParameterizedTypeReference<List<ChengPinRuKu>>() {}
        );
        List<ChengPinRuKu> forObjects1= response1.getBody();
        System.out.println(forObjects1);
        for (ChengPinRuKu forObject : forObjects1) {

            ErpBdOrg forgid = erpBdOrgMapper.selectOne(new QueryWrapper<ErpBdOrg>().eq("forgid", forObject.getFSTOCKORGID()));
            if (ObjectUtil.isEmpty(forgid))
                continue;
            WarehouseIncomStock warehouseIncomStock  = new WarehouseIncomStock();
            //原材料
            if (forObject.getFnumber().startsWith("10") && !forObject.getFnumber().equals("10020002")){
                 warehouseIncomStock.setMaterial(String.format("%.2f",forObject.getFREALQTY()));
            }
            //成品、半成品
            if (forObject.getFnumber().startsWith("01") ){
                 warehouseIncomStock.setFinishProduce(String.format("%.2f",forObject.getFREALQTY()));
            }
            //低值易耗品
            if (forObject.getFnumber().startsWith("15")){
                 warehouseIncomStock.setLowValue(String.format("%.2f",forObject.getFREALQTY()));
            }
            //包材
            if (forObject.getFnumber().startsWith("11")){
                 warehouseIncomStock.setPackageMaterial(String.format("%.2f",forObject.getFREALQTY()));
            }
            //羊毛脂
            if ("10020002".equals(forObject.getFnumber())){
                 warehouseIncomStock.setLanolin(String.format("%.2f",forObject.getFREALQTY()));
            }
            //鱼油
            if ("10020071".equals(forObject.getFnumber())){
                 warehouseIncomStock.setFishOil(String.format("%.2f",forObject.getFREALQTY()));
            }

            warehouseIncomStock.setCreateTime(forObject.getFDATE());
            warehouseIncomStock.setOrgId(forgid.getOrgId());
            incomStockMapper.insert(warehouseIncomStock);

            WarehouseMaterialInStock warehouseMaterialInStock = new WarehouseMaterialInStock();
            warehouseMaterialInStock.setCreateTime(forObject.getFDATE());
            warehouseMaterialInStock.setFnumber(forObject.getFnumber());
            warehouseMaterialInStock.setMaterial(forObject.getWuLiaoName());
            warehouseMaterialInStock.setMaterialType(getmater(forObject.getFnumber()));
            warehouseMaterialInStock.setOrgId(forgid.getOrgId());
            warehouseMaterialInStock.setOutStock(String.format("%.2f",forObject.getFREALQTY()));
            warehouseMaterialInStock.setOrgName(forgid.getFname());
            materialInStockMapper.insert(warehouseMaterialInStock);

            WarehouseStockDetail warehouseStockDetail = new WarehouseStockDetail();

            if (forObject.getFnumber().startsWith("10") && !forObject.getFnumber().equals("10020002")){
                warehouseStockDetail.setMaterialCode(0);
            }
            //成品、半成品
            if (forObject.getFnumber().startsWith("01") ){
                warehouseStockDetail.setMaterialCode(1);
            }
            //低值易耗品
            if (forObject.getFnumber().startsWith("15")){
                warehouseStockDetail.setMaterialCode(3);
            }
            //包材
            if (forObject.getFnumber().startsWith("11")){
                warehouseStockDetail.setMaterialCode(4);
            }
            //羊毛脂
            if ("10020002".equals(forObject.getFnumber())){
                warehouseStockDetail.setMaterialCode(5);
            }

            if ("10020071".equals(forObject.getFnumber())){
                warehouseStockDetail.setMaterialCode(6);
            }

            warehouseStockDetail.setOrgId(forgid.getOrgId());
            warehouseStockDetail.setMaterialName(forObject.getWuLiaoName());
            warehouseStockDetail.setIncomeNumber(String.format("%.2f",forObject.getFREALQTY()));
            warehouseStockDetail.setStockName(forObject.getFNAME());
            warehouseStockDetail.setCreateTime(forObject.getFDATE());
            warehouseStockDetail.setFnumber(forObject.getFnumber());
            stockDetailMapper.insert(warehouseStockDetail);
        }
    }

    @Override
    public void getChengPinChuKu(Date date) {
        ResponseEntity<List<ChenPinChuKu>> response1 = restTemplate.exchange(
                "http://127.0.0.1:9200/erp-kucun/getChengPinChuKu?date="+date,
                HttpMethod.GET,
                null,
                new ParameterizedTypeReference<List<ChenPinChuKu>>() {}
        );
        List<ChenPinChuKu> forObjects1 = response1.getBody();
        System.out.println(forObjects1);

        for (ChenPinChuKu forObject : forObjects1) {

            ErpBdOrg forgid = erpBdOrgMapper.selectOne(new QueryWrapper<ErpBdOrg>().eq("forgid", forObject.getFSTOCKORGID()));
            if (ObjectUtil.isEmpty(forgid))
                continue;
            WarehouseMaterialOutStock warehouseMaterialOutStock = new WarehouseMaterialOutStock();
            warehouseMaterialOutStock.setCreateTime(forObject.getFDATE());
            warehouseMaterialOutStock.setFnumber(forObject.getFnumber());
            warehouseMaterialOutStock.setMaterial(forObject.getWuLiaoName());
            warehouseMaterialOutStock.setMaterialType(getmater(forObject.getFnumber()));
            warehouseMaterialOutStock.setOrgId(forgid.getOrgId());
            warehouseMaterialOutStock.setOutStock(String.format("%.2f",forObject.getFSALUNITQTY()));
            warehouseMaterialOutStock.setOrgName(forgid.getFname());
            materialOutStockMapper.insert(warehouseMaterialOutStock);

            WarehouseStockDetail warehouseStockDetail = new WarehouseStockDetail();
            if (forObject.getFnumber().startsWith("10") && !forObject.getFnumber().equals("10020002")){
                warehouseStockDetail.setMaterialCode(0);
            }
            //成品、半成品
            if (forObject.getFnumber().startsWith("01") ){
                warehouseStockDetail.setMaterialCode(1);
            }
            //低值易耗品
            if (forObject.getFnumber().startsWith("15")){
                warehouseStockDetail.setMaterialCode(3);
            }
            //包材
            if (forObject.getFnumber().startsWith("11")){
                warehouseStockDetail.setMaterialCode(4);
            }
            //羊毛脂
            if ("10020002".equals(forObject.getFnumber())){
                warehouseStockDetail.setMaterialCode(5);
            }

            if ("10020071".equals(forObject.getFnumber())){
                warehouseStockDetail.setMaterialCode(6);
            }
            warehouseStockDetail.setOrgId(forgid.getOrgId());
            warehouseStockDetail.setMaterialName(forObject.getWuLiaoName());
            warehouseStockDetail.setOutNumber(String.format("%.2f",forObject.getFSALUNITQTY()));
            warehouseStockDetail.setStockName(forObject.getFNAME());
            warehouseStockDetail.setCreateTime(forObject.getFDATE());
            stockDetailMapper.insert(warehouseStockDetail);

        }
    }
      @Override
    public void getWuLiaoMingXi(Date date) {
        ResponseEntity<List<WuLiaoKuMingXi>> response = restTemplate.exchange(
                "http://127.0.0.1:9200/erp-kucun/getWuLiaoMingXi?date="+date,
                HttpMethod.GET,
                null,
                new ParameterizedTypeReference<List<WuLiaoKuMingXi>>() {}
        );
        List<WuLiaoKuMingXi> forObjects = response.getBody();
        for (WuLiaoKuMingXi forObject : forObjects) {

            ErpBdOrg forgid = erpBdOrgMapper.selectOne(new QueryWrapper<ErpBdOrg>().eq("forgid", forObject.getFSTOCKORGID()));
            if (ObjectUtil.isEmpty(forgid))
                continue;
            WarehouseStockSecond warehouseStockSecond = new WarehouseStockSecond();
            if (("原料库".equals(forObject.getFNAME()) || "高科原料仓库".equals(forObject.getFNAME())) && !forObject.getFnumber().equals("10020002")){
                warehouseStockSecond.setMaterialType(0);
            }
            if ("成品库".equals(forObject.getFNAME()) || "成品库CW".equals(forObject.getFNAME())){
                warehouseStockSecond.setMaterialType(1);
            }
            if ("五金库".equals(forObject.getFNAME())){
                warehouseStockSecond.setMaterialType(3);
            }
            if ("包材库".equals(forObject.getFNAME())){
                warehouseStockSecond.setMaterialType(4);
            }
            if (forObject.getFnumber().equals("10020002")){
                warehouseStockSecond.setMaterialType(5);
            }
            if ("10020071".equals(forObject.getFnumber())){
                warehouseStockSecond.setMaterialType(6);
            }
            warehouseStockSecond.setIncomeNumber(String.format("%.2f",forObject.getFINSTOCKQTY()));
            warehouseStockSecond.setIncomeMoney(String.format("%.2f",forObject.getFBILLAMOUNTLC()));
            warehouseStockSecond.setCreateTime(forObject.getFDATE());
            warehouseStockSecond.setOrgId(forgid.getOrgId());
            warehouseStockSecond.setFnumber(forObject.getFnumber());
            warehouseStockSecond.setStock(String.format("%.2f",forObject.getFSTOCKBASEQTY()));
            stockSecondMapper.insert(warehouseStockSecond);
        }
    }

    private Integer getmater(String fname) {
        if (fname.startsWith("10") && !fname.equals("10020002") && !fname.equals("10020071") ){
            return 0;
        }
        //成品、半成品
        if (fname.startsWith("01") ){
            return 1;
        }
        //低值易耗品
        if (fname.startsWith("15")){
            return 2;
        }
        //包材
        if (fname.startsWith("11")){
            return 3;
        }
        //羊毛脂
        if (fname.equals("10020002")){
            return 5;
        }

        if (fname.equals("10020071")){
            return 6;
        }
        return 7;
    }

    private Integer getMaterialType(String fname) {
        if (("原料库".equals(fname) || "高科原料仓库".equals(fname))){
            return 0;
        }
        if ("成品库".equals(fname) || "成品库CW".equals(fname)){
            return 1;
        }
        if ("五金库".equals(fname)){
            return 2;
        }
        if ("包材库".equals(fname)){
            return 3;
        }
        return 0;
    }


}
