package vip.xiaonuo.importData.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.druid.sql.visitor.functions.If;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.hr.modular.basedepartment.entity.HrBaseDepartment;
import vip.xiaonuo.hr.modular.basedepartment.service.HrBaseDepartmentService;
import vip.xiaonuo.hr.modular.baseemployeeinfo.entity.HrBaseEmployeeInfo;
import vip.xiaonuo.hr.modular.baseemployeeinfo.service.HrBaseEmployeeInfoService;
import vip.xiaonuo.importData.entity.*;
import vip.xiaonuo.importData.service.ImportData;
import vip.xiaonuo.importData.utils.ExcelImportSheet;
import vip.xiaonuo.money.entity.*;
import vip.xiaonuo.money.entity.vo.MoneyVo;
import vip.xiaonuo.money.mapper.*;
import vip.xiaonuo.sys.modular.org.entity.SysOrg;
import vip.xiaonuo.sys.modular.org.mapper.SysOrgMapper;

import javax.servlet.http.HttpServletRequest;
import java.io.InputStream;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static vip.xiaonuo.common.pojo.CommonResult.CODE_ERROR;

@Service
public class ImportDataImpl implements ImportData {

    @Autowired
    private HrBaseDepartmentService departmentService;
    @Autowired
    private HrBaseEmployeeInfoService infoService;
    @Autowired
    private MoneyCostMapper moneyCostMapper;
    @Autowired
    private SysOrgMapper orgMapper;
    @Autowired
    private MoneyWriteOverdueMapper overdueMapper;
    @Autowired
    private MoneyBusinessMapper businessMapper;
    @Autowired
    private MoneyBorrowWarningMapper moneyBorrowWarningMapper;
    @Autowired
    private MoneyProduce0422Mapper moneyProduce0422Mapper;


    @Override
    @Transactional
    public CommonResult importData(HttpServletRequest httpServletRequest, String orgId, MultipartFile files) {
        Map<String , String> map = new HashMap<>();
        //表头与键值对的映射关系
        map.put("部门" , "departmentName");
        map.put("岗位" , "post");
        map.put("序号" , "number");
        map.put("姓名" , "name");
        map.put("性别" , "gender");
        map.put("政治面貌" , "politicalOutlook");
        map.put("民族" , "nation");
        map.put("户口性质" , "hukouNature");
        map.put("出生年月" , "birthday");
        map.put("年龄(周岁)" , "age");
        map.put("身份证号码" , "idCardNumber");
        map.put("招入集团时间" , "joinGroup");
        map.put("入职时间" , "entryDate");
        map.put("工龄" , "seniority");
        map.put("合同起止时间" , "contactStartTime");
        map.put("文化程度" , "cultureLevelNum");
        map.put("专业" , "major");
        map.put("毕业院校" , "almaMater");
        map.put("职称" , "title");
        map.put("现任职务或岗位" , "current");
        map.put("新级别" , "newLevel");
        map.put("新薪档" , "newSalary");
        map.put("转正日期" , "confirmationTime");
        map.put("定额工资" , "quotaWages");
        map.put("籍贯" , "nativePlace");
        map.put("家庭地址或通讯地址" , "homeAddress");
        map.put("手机" , "phone");
        map.put("紧急联系人姓名" , "emergencyContactName");
        map.put("紧急联系人电话" , "emergencyContactPhone");
        map.put("工资发放处" , "payroll");
        map.put("社保" , "socialSecurity");
        map.put("公积金" , "providentFund");
        map.put("婚姻" , "marriage");
        map.put("是否退休返聘" , "tuixiu");
        String idCard = "";
        try(
                //这里面的对象会自动关闭
                InputStream in = files.getInputStream();
                Workbook workbook = ExcelImportSheet.getTypeFromExtends(in , files.getOriginalFilename())
        ) {

            //根据名称获取单张表对象 也可以使用getSheetAt(int index)获取单张表的对象 获取第一张表
            Sheet sheet = workbook.getSheetAt(0);
            List<HrData> list = ExcelImportSheet.getListFromExcel(sheet , HrData.class , map);

            for (HrData hrData : list) {
                String department = hrData.getDepartmentName();
                System.out.println("这个是部门的名称："+department);
                if ("".equals(department) || null == department) {
                } else {
                    HrBaseDepartment departments = departmentService.getOne(new QueryWrapper<HrBaseDepartment>().eq("name", department).eq("org_id",orgId));
                    if (ObjectUtil.isNotEmpty(departments)) {
                    } else {
                        HrBaseDepartment department1 = new HrBaseDepartment();
                        department1.setOrgId(orgId);
                        department1.setName(department);
                        department1.setCreateTime(new Date());
                        departmentService.save(department1);
                    }
                }
            }
            List<HrBaseDepartment> departments1 = departmentService.list(new QueryWrapper<HrBaseDepartment>().eq("org_id", orgId));

            for (HrData hrData : list) {
                HrBaseEmployeeInfo info = new HrBaseEmployeeInfo();
                BeanUtil.copyProperties(hrData, info);
                idCard  = hrData.getIdCardNumber();
                System.out.println(idCard);
                info.setEducation(hrData.getCultureLevelNum());
                info.setCultureLevel(getCultureLevel(hrData.getCultureLevelNum()));


                info.setOrgId(orgId);
                for (HrBaseDepartment department : departments1) {
                    if (department.getName().equals(hrData.getDepartmentName())) {
                        info.setDepartmentName(department.getName());
                        info.setDepartmentId(department.getId().toString());
                    }
                }
                String seniority = hrData.getSeniority();
                info.setSeniority(Integer.parseInt(seniority.split("年")[0]));


                if ("男".equals(hrData.getGender()) || "男性".equals(hrData.getGender())) {
                    info.setGender("0");
                }
                if ("女".equals(hrData.getGender()) || "女性".equals(hrData.getGender())) {
                    info.setGender("1");
                }
                if ("已婚".equals(hrData.getMarriage())) {
                    info.setMarriage("1");
                }
                if ("未婚".equals(hrData.getMarriage())) {
                    info.setMarriage("0");
                }
                String contactStartTime = hrData.getContactStartTime();
                if (ObjectUtil.isNotEmpty(contactStartTime)){
                    info.setSiLing(getSiLing(contactStartTime));
                    info.setUserType(getEndDate(contactStartTime));
                }
                Date birthday = hrData.getBirthday();
                info.setRetireTime(DateUtil.offset(birthday, DateField.YEAR,60));
                info.setRetier(60 > Integer.parseInt(info.getAge()) ? 0 : 1);
                if ("".equals(idCard) || null == idCard) {
                    return CommonResult.error("表格中存在身份证号为空！请检查后重新导入");
                } else {
                    HrBaseEmployeeInfo infos = infoService.getOne(new QueryWrapper<HrBaseEmployeeInfo>().eq("ID_CARD_NUMBER", hrData.getIdCardNumber()));
                    System.out.println(info.toString());
                    if (ObjectUtil.isEmpty(infos)) {
                        infoService.save(info);
                    } else {
                        info.setId(infos.getId());
                        infoService.updateById(info);
                    }
                }
            }

        } catch (Exception e) {
            e.printStackTrace();
            return new CommonResult<>(CODE_ERROR, String.format("抱歉,身份证号为%s的人员数据有误,", idCard), null);
        }finally {
            //写着好看的
        }
        return CommonResult.ok();
    }

    private Integer getSiLing(String contactStartTime) {
        String startDateStr = contactStartTime.split("-")[0];

        // 定义日期格式
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy.MM.dd");

        // 解析结束日期
        LocalDate startDate = LocalDate.parse(startDateStr, formatter);
        // 获取当前日期
        LocalDate currentDate = LocalDate.now();
        return  (int)ChronoUnit.YEARS.between(startDate, currentDate );


    }



    public  Integer getEndDate(String strt){
        String startDateStr = strt.split("-")[0];
        // 提取结束日期
        String endDateStr = strt.split("-")[1];

        // 定义日期格式
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy.MM.dd");
        System.out.println("______>"+strt);
        // 解析结束日期
        endDateStr =  setLing(endDateStr);
        startDateStr =setLing(startDateStr);
        LocalDate endDate = LocalDate.parse(endDateStr, formatter);
        LocalDate startDate = LocalDate.parse(startDateStr, formatter);
        // 获取当前日期
        LocalDate currentDate = LocalDate.now();
        if(currentDate.isBefore(startDate)){
            return 3;
        }
        // 比较日期
        if (currentDate.isBefore(endDate)) {
            return  0;
        } else  {
            return 1;
        }
    }

    private String setLing(String endDateStr) {
        String[] split = endDateStr.split("\\.");
        if (endDateStr.length() != 10) {
            if (split[1].length() == 1) {
                split[1] = "0" + split[1];
            }
            if (split[2].length() == 1) {
                split[2] = "0" + split[2];
            }
            return split[0]+"."+split[1]+"."+split[2];
        }
        return endDateStr;
    }

    private Integer getCultureLevel(String cultureLevel) {
        //学历 0小学 1初中  2高中  4大专 研究生  博士"
        if (cultureLevel.contains("小学")){
            return 0;
        }else if (cultureLevel.contains("初中")){
            return 1;
        }else if (cultureLevel.contains("高中")){
            return 2;
        }else if (cultureLevel.contains("大专")){
            return 4;
        }else if (cultureLevel.contains("本科")){
            return 5;
        }else if (cultureLevel.contains("研究生")){
            return 6;
        }else if (cultureLevel.contains("博士")){
            return 7;
        }else {
            return 8;
        }
    }


    @Override
    public CommonResult importMoney(HttpServletRequest httpServletRequest, String orgId, MultipartFile files) {
        Map<String , String> map = new HashMap<>();
        //表头与键值对的映射关系
        map.put("填报时间" , "createTime");
        map.put("管理费用" , "manage");
        map.put("销售费用" , "sale");
        map.put("财务费用" , "finance");
        SysOrg org = orgMapper.selectOne(new QueryWrapper<SysOrg>().eq("id", orgId));
        try(
                //这里面的对象会自动关闭
                InputStream in = files.getInputStream();
                Workbook workbook = ExcelImportSheet.getTypeFromExtends(in , files.getOriginalFilename())
        ) {

            //根据名称获取单张表对象 也可以使用getSheetAt(int index)获取单张表的对象 获取第一张表
            Sheet sheet = workbook.getSheetAt(0);
            List<MoneyData> list = ExcelImportSheet.getListFromExcel(sheet , MoneyData.class , map);

            for (MoneyData moneyData : list) {
                if (moneyData.getCreateTime() == null) {
                } else {
                    MoneyCost moneyCost = new MoneyCost();
                    moneyCost.setSale(getFloat(moneyData.getSale()));
                    moneyCost.setManage(getFloat(moneyData.getManage()));
                    moneyCost.setFinance(getFloat(moneyData.getFinance()));
                    moneyCost.setCreateTime(moneyData.getCreateTime());
                    moneyCost.setOrgId(orgId);
                    moneyCost.setCompany(org.getName());
                    moneyCostMapper.insert(moneyCost);
                }

            }

        } catch (Exception e) {
            e.printStackTrace();
            return CommonResult.error();
        }finally {
            //写着好看的
        }
        return CommonResult.ok();
    }

    private static String getFloat(String sale) {

        if (StringUtils.isNotBlank(sale)){
            return String.format("%.2f",Double.parseDouble(sale));
        }else {
            return "0";
        }
    }

    @Override
    @Transactional
    public CommonResult importOverdue(HttpServletRequest httpServletRequest, String orgId, MultipartFile files) {
        Map<String , String> map = new HashMap<>();
        //表头与键值对的映射关系
//        map.put("单据编号" , "documentNumber");
//        map.put("单据类型" , "documentType");
//        map.put("单据状态" , "documentStatus");
//
//        map.put("创建时间" , "createTime");
//        map.put("到期日" , "endTime");
//        map.put("逾期金额" , "money");

        map.put("客户名称" , "userName");
        map.put("币种" , "bz");
        map.put("原币金额" , "ybje");
        map.put("折人民币金额" , "rmbje");
        map.put("销售日期" , "xsrq");
        map.put("收款期限" , "skqx");
        map.put("到期日期" , "endTime");
        map.put("逾期天数" , "yqts");
        map.put("合同号" , "documentNumber");
        map.put("截止日期" , "deadline");


//        SysOrg org = orgMapper.selectOne(new QueryWrapper<SysOrg>().eq("id", orgId));
        try(
                //这里面的对象会自动关闭
                InputStream in = files.getInputStream();
                Workbook workbook = ExcelImportSheet.getTypeFromExtends(in , files.getOriginalFilename())
        ) {

            //根据名称获取单张表对象 也可以使用getSheetAt(int index)获取单张表的对象 获取第一张表
            Sheet sheet = workbook.getSheetAt(0);
            List<OverdueData> list = ExcelImportSheet.getListFromExcel(sheet , OverdueData.class , map);
            for (OverdueData overdueData : list) {
                if (StringUtils.isNotBlank( overdueData.getDocumentNumber())) {
                    MoneyWriteOverdue overdue = new MoneyWriteOverdue();
                    overdue.setOrgId(orgId);
//                    overdue.setCreateTime(overdueData.getCreateTime());
//                    overdue.setDocumentNumber(overdueData.getDocumentNumber());
//                    overdue.setDocumentType(overdueData.getDocumentType());
//                    overdue.setDocumentStatus(overdueData.getDocumentStatus());
//                    overdue.setEndTime(overdueData.getEndTime());
//                    overdue.setMoney(overdueData.getMoney());
//                    overdue.setUserName(overdueData.getUserName());

                    overdue.setUserName(overdueData.getUserName());
                    overdue.setBz(overdueData.getBz());
                    overdue.setYbje(getFloat(overdueData.getYbje()));
                    overdue.setRmbje(getFloat(overdueData.getRmbje()));
                    overdue.setXsrq(overdueData.getXsrq());
                    overdue.setSkqx(overdueData.getSkqx());
                    overdue.setEndTime(overdueData.getEndTime());
                    overdue.setYqts(overdueData.getYqts());
                    overdue.setDeadline(overdueData.getDeadline());
                    overdue.setDocumentNumber(overdueData.getDocumentNumber());
                    overdue.setBatchLast(1);
                    overdueMapper.insert(overdue);
                }

            }
            MoneyWriteOverdue moneyWriteOverdue = new MoneyWriteOverdue();
            moneyWriteOverdue.setBatchLast(0);
            overdueMapper.update(moneyWriteOverdue, new UpdateWrapper<MoneyWriteOverdue>().eq("document_type",1));
            System.out.println("批量修改完成");
        } catch (Exception e) {
            e.printStackTrace();
            return CommonResult.error();
        }finally {
            //写着好看的
        }
        return CommonResult.ok();
    }

    @Override
    public CommonResult importBalance(HttpServletRequest httpServletRequest, String orgId, MultipartFile files) {
        Map<String , String> map = new HashMap<>();
        //表头与键值对的映射关系
//        map.put("填报公司" , "orgName");
        map.put("资金余额" , "balance");
        map.put("创建时间" , "createTime");
        map.put("截止日期" , "deadline");



//        SysOrg org = orgMapper.selectOne(new QueryWrapper<SysOrg>().eq("id", orgId));
        try(
                //这里面的对象会自动关闭
                InputStream in = files.getInputStream();
                Workbook workbook = ExcelImportSheet.getTypeFromExtends(in , files.getOriginalFilename())
        ) {

            //根据名称获取单张表对象 也可以使用getSheetAt(int index)获取单张表的对象 获取第一张表
            Sheet sheet = workbook.getSheetAt(0);
            List<BalanceData> list = ExcelImportSheet.getListFromExcel(sheet , BalanceData.class , map);
            for (BalanceData balanceData : list) {
                if (null == balanceData.getCreateTime()) {
                } else {
                    MoneyBusiness moneyBusiness = new MoneyBusiness();
                    moneyBusiness.setOrgId(orgId);
                    moneyBusiness.setBalance(getFloat(balanceData.getBalance()));
                    moneyBusiness.setCreateTime(balanceData.getCreateTime());
                    moneyBusiness.setDeadline(balanceData.getDeadline());
                    businessMapper.insert(moneyBusiness);
                }

            }

        } catch (Exception e) {
            e.printStackTrace();
            return CommonResult.error();
        }finally {
            //写着好看的
        }
        return CommonResult.ok();
    }

    @Override
    public CommonResult importBank(HttpServletRequest httpServletRequest, String orgId, MultipartFile files) {
        Map<String , String> map = new HashMap<>();
        //表头与键值对的映射关系

        map.put("借款时间" , "borrowTime");
        map.put("币种" , "moneyType");
        map.put("借款金额(万元)" , "borrowPrice");
        map.put("借款银行" , "bankName");
        map.put("借款利率(%)" , "bankRank");
        map.put("借款类型" , "bankType");
        map.put("还款时间" , "repaymentTime");
        map.put("截止日期" , "deadline");

        SysOrg org = orgMapper.selectOne(new QueryWrapper<SysOrg>().eq("id", orgId));
        try(
                //这里面的对象会自动关闭
                InputStream in = files.getInputStream();
                Workbook workbook = ExcelImportSheet.getTypeFromExtends(in , files.getOriginalFilename())
        ) {

            //根据名称获取单张表对象 也可以使用getSheetAt(int index)获取单张表的对象 获取第一张表
            Sheet sheet = workbook.getSheetAt(0);
            List<BankData> list = ExcelImportSheet.getListFromExcel(sheet , BankData.class , map);
            for (BankData bankData : list) {
                if (StringUtils.isNotBlank( bankData.getBankName())) {
                    MoneyBorrowWarning moneyBorrowWarning = new MoneyBorrowWarning();
                    moneyBorrowWarning.setOrgId(orgId);
                    moneyBorrowWarning.setCompany(org.getName());
                    moneyBorrowWarning.setMoneyType(bankData.getMoneyType());
                    moneyBorrowWarning.setBorrowPrice(getFloat(bankData.getBorrowPrice()));
                    moneyBorrowWarning.setBorrowTime(bankData.getBorrowTime());
                    moneyBorrowWarning.setBankName(bankData.getBankName());
                    moneyBorrowWarning.setBankRank(getFloat(bankData.getBankRank()));
                    moneyBorrowWarning.setBankType(bankData.getBankType());
                    moneyBorrowWarning.setEndTime(bankData.getEndTime());
                    moneyBorrowWarning.setRepaymentTime(bankData.getRepaymentTime());
                    moneyBorrowWarning.setDeadline(bankData.getDeadline());

                    moneyBorrowWarningMapper.insert(moneyBorrowWarning);
                }

            }

        } catch (Exception e) {
            e.printStackTrace();
            return CommonResult.error();
        }finally {
            //写着好看的
        }
        return CommonResult.ok();
    }

    @Override
    public CommonResult importRemove(HttpServletRequest httpServletRequest, String orgId, MultipartFile files) {

        Map<String , String> map = new HashMap<>();
        //表头与键值对的映射关系
        map.put("序号" , "num");
        map.put("填报时间" , "createTime");

        map.put("交易关联方", "customerName");
        map.put("物料名称" , "materialName");
        map.put("金额" , "materialAmount");

        SysOrg org = orgMapper.selectOne(new QueryWrapper<SysOrg>().eq("id", orgId));
        try(
                //这里面的对象会自动关闭
                InputStream in = files.getInputStream();
                Workbook workbook = ExcelImportSheet.getTypeFromExtends(in , files.getOriginalFilename())
        ) {

            //根据名称获取单张表对象 也可以使用getSheetAt(int index)获取单张表的对象 获取第一张表
            Sheet sheet = workbook.getSheetAt(0);
            List<RemoveData> list = ExcelImportSheet.getListFromExcel(sheet , RemoveData.class , map);
            MoneyProduce0422 moneyProduce0422 = moneyProduce0422Mapper.selectOne(new QueryWrapper<MoneyProduce0422>().isNotNull("write_money").orderByDesc("id").last("limit 1"));
            Integer num = 1;
            if (ObjectUtil.isNotEmpty(moneyProduce0422) && StringUtils.isNotBlank(moneyProduce0422.getNum())){
                num =Integer.parseInt(moneyProduce0422.getNum().split("-")[0]) +1 ;
            }
            for (RemoveData bankData : list) {
                if (StringUtils.isNotBlank( bankData.getMaterialAmount())) {
                    MoneyProduce0422 produce0422= new MoneyProduce0422();
                    produce0422.setOrgId(orgId);
                    produce0422.setCreateTime(bankData.getCreateTime());
                    produce0422.setMaterialName(bankData.getMaterialName());
                    produce0422.setMaterialType(getMaterType(bankData.getMaterialName()));
                    produce0422.setMaterialAmount(String.format("%.2f", Double.parseDouble(  bankData.getMaterialAmount()) * -1.0));
                    produce0422.setWriteMoney(getFloat(bankData.getMaterialAmount()));
                    produce0422.setOrgName(org.getName());
                    produce0422.setCustomerName(bankData.getCustomerName());
                    produce0422.setIsWrite(1);
                    produce0422.setNum(num +"-" + bankData.getNum());
                    moneyProduce0422Mapper.insert(produce0422);
                }

            }

        } catch (Exception e) {
            e.printStackTrace();
            return CommonResult.error();
        }finally {
            //写着好看的
        }
        return CommonResult.ok();

    }



    private Integer getMaterType(String fname) {
        if ("NF级胆固醇".equals(fname)){
            return 0;
        }
        if ("羊毛酸异丙酯".equals(fname)){
            return 4;
        }
        if (fname.contains( "饲料级胆固醇")){
            return 1;
        }
        if (fname.contains( "羊毛醇")){
            return 2;
        }
        if (fname.contains( "羊毛酸")){
            return 3;
        }
        return 5;
    }
}
