package vip.xiaonuo.money.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import vip.xiaonuo.money.entity.MoneyCost;
import vip.xiaonuo.money.entity.MoneyProducePrice;
import vip.xiaonuo.money.entity.vo.MoneyBorrowPriceVo;
import vip.xiaonuo.money.entity.vo.MoneyProduceVo;
import vip.xiaonuo.money.mapper.MoneyCostMapper;
import vip.xiaonuo.money.mapper.MoneyProducePriceMapper;
import vip.xiaonuo.money.param.MoneyInOutParam;
import vip.xiaonuo.money.service.IMoneyProducePriceService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import vip.xiaonuo.sale.entity.SaleCost;
import vip.xiaonuo.sale.entity.SaleDetail;
import vip.xiaonuo.sale.mapper.SaleCostMapper;
import vip.xiaonuo.sale.mapper.SaleDetailMapper;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 财务产品成本收入 服务实现类
 * </p>
 *
 * @author author
 * @since 2024-11-21
 */
@Service
public class MoneyProducePriceServiceImpl extends ServiceImpl<MoneyProducePriceMapper, MoneyProducePrice> implements IMoneyProducePriceService {

    @Autowired
    private MoneyProducePriceMapper producePriceMapper;

    @Autowired
    private SaleDetailMapper detailMapper;
    @Autowired
    private SaleCostMapper costMapper;
    @Autowired
    private MoneyCostMapper moneyCostMapper;


    @Override
    public List<MoneyProducePrice> listProduceProfit(MoneyInOutParam outParam) {
        QueryWrapper<MoneyProducePrice> queryWrapper = new QueryWrapper<>();
        DateTime parse = new DateTime();
        if (StringUtils.isNotBlank(outParam.getDate())){
            parse = DateUtil.parse(outParam.getDate(), "yyyy-MM");
        }

        DateTime dateTime = DateUtil.offsetMonth(parse, -12);
        queryWrapper.eq(StringUtils.isNotBlank(outParam.getOrgId()), "org_id", outParam.getOrgId());
        queryWrapper.like(StringUtils.isNotBlank(outParam.getChanPin()), "produce_name", outParam.getChanPin());
        queryWrapper.ge("create_time",dateTime);

        return producePriceMapper.listProduceProfit(queryWrapper);

    }

    @Override
    public List<MoneyProduceVo> listProduce(MoneyInOutParam outParam) {
        String orgId = outParam.getOrgId();
        String format = outParam.getDate();

        if (StringUtils.isBlank(format)){
            format=DateUtil.format(new Date(),"yyyy-MM");
        }
        DateTime now = DateUtil.parse(format, "yyyy-MM");

        List<MoneyProduceVo> list = new ArrayList<>();
        BigDecimal math = new BigDecimal("10000");
        for (int i = 11; i >=0; i--) {
            MoneyProduceVo vo = new MoneyProduceVo();
            QueryWrapper<MoneyCost> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
            queryWrapper.select(" IFNULL(sum(convert(sale,decimal(10,2))),'0') sale");
            QueryWrapper<SaleDetail> queryWrapper1 = new QueryWrapper<>();
            queryWrapper1.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
            queryWrapper1.select(" IFNULL(sum(convert(sale_money,decimal(10,2))),'0') saleMoney");
            DateTime dateTime = DateUtil.offsetMonth(now, -i);

            queryWrapper.between("create_time",dateTime,DateUtil.endOfMonth(dateTime));
            queryWrapper1.between("create_time",dateTime,DateUtil.endOfMonth(dateTime));
            vo.setTime(DateUtil.format(dateTime,"yyyy-MM"));
            MoneyCost saleCost = moneyCostMapper.selectOne(queryWrapper);//销售成本
            SaleDetail saleDetail = detailMapper.selectOne(queryWrapper1);//销售额
            if (ObjectUtil.isEmpty(saleCost)) {
                vo.setCost("0");
            } else {
                BigDecimal bg = new BigDecimal(saleCost.getSale());
                bg = bg.divide(math, 2, RoundingMode.HALF_UP);
                vo.setCost(bg.toString());
            }
            if (ObjectUtil.isEmpty(saleDetail)) {
                vo.setIncome("0");
            } else {
                BigDecimal bg = new BigDecimal(saleDetail.getSaleMoney());
                bg = bg.divide(math, 2, RoundingMode.HALF_UP);
                vo.setIncome(bg.toString());
            }
            list.add(vo);
        }

        return list;
    }

}
