package vip.xiaonuo.purchase.service.impl;

import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.models.auth.In;
import org.springframework.beans.factory.annotation.Autowired;
import vip.xiaonuo.erp.entity.ErpBdRawMaterial;
import vip.xiaonuo.erp.mapper.ErpBdRawMaterialMapper;
import vip.xiaonuo.money.entity.MoneyBusiness;
import vip.xiaonuo.money.param.DsBusAllParam;
import vip.xiaonuo.purchase.entity.PurchaseOrder;
import vip.xiaonuo.purchase.entity.PurchasePrice;
import vip.xiaonuo.purchase.entity.vo.*;
import vip.xiaonuo.purchase.mapper.PurchaseOrderMapper;
import vip.xiaonuo.purchase.mapper.PurchasePriceMapper;
import vip.xiaonuo.purchase.param.*;
import vip.xiaonuo.purchase.service.IPurchaseOrderService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import vip.xiaonuo.quality.entity.QualityWarning;
import vip.xiaonuo.warehouse.entity.WarehouseStockDetail;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.YearMonth;
import java.time.temporal.ChronoUnit;
import java.util.*;

import static vip.xiaonuo.money.service.impl.MoneyBusinessServiceImpl.calculateYOYChange;
import static vip.xiaonuo.money.service.impl.MoneyBusinessServiceImpl.calculateYOYZiJinYvE;

/**
 * <p>
 * 采购订单表 服务实现类
 * </p>
 *
 * @author author
 * @since 2024-11-25
 */
@Service
public class PurchaseOrderServiceImpl extends ServiceImpl<PurchaseOrderMapper, PurchaseOrder> implements IPurchaseOrderService {

    @Autowired
    private PurchaseOrderMapper purchaseOrderMapper;
    @Autowired
    private PurchasePriceMapper purchasePriceMapper;
    @Autowired
    private ErpBdRawMaterialMapper erpBdRawMaterialMapper;



    public static void main(String[] args) {
        System.out.println(DateUtil.parse("2022-01", "yyyy-MM"));
    }
    @Override
    public PurchaseOrderVo getOrder(PurchaseParam purchaseParam) {
        String orgId = purchaseParam.getOrgId();
//        String time = purchaseParam.getTime();
//        DateTime dateTime = DateUtil.beginOfYear(new Date());
        System.out.println(purchaseParam.getTime());

        DateTime now = DateUtil.parse(purchaseParam.getTime(), "yyyy-MM");
        DateTime lastYear = DateUtil.offsetMonth(now, -12);

        QueryWrapper<PurchaseOrder> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId).orderByAsc("create_time");
        queryWrapper.between("create_time",DateUtil.beginOfMonth(now),DateUtil.endOfMonth(now));
        PurchaseOrderVo  test =purchaseOrderMapper.getOrder(queryWrapper);
        queryWrapper.clear();
        queryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId).orderByAsc("create_time");
        queryWrapper.between("create_time",DateUtil.beginOfMonth(lastYear),DateUtil.endOfMonth(lastYear));
        PurchaseOrderVo lastTest = purchaseOrderMapper.getOrder(queryWrapper);
        test.setMonthRank(  calculateYOYZiJinYvE(getDouble(test.getPurchaseMoney()), getDouble(lastTest.getPurchaseMoney())));

        queryWrapper.clear();
        queryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId).orderByAsc("create_time");
        queryWrapper.between("create_time",DateUtil.beginOfYear(lastYear),DateUtil.endOfYear(lastYear));
        PurchaseOrderVo lastTestYear = purchaseOrderMapper.getOrder(queryWrapper);

        test.setLastAvgPrice(String.format("%.2f",getDouble(lastTestYear.getPurchaseMoney())/12));
        test.setRank("98");
        return test;
    }

    @Override
    public List<PurchaseOrderMoneyVo> listOrderMoney(PurchaseOrderParam orderParam) {
        String orgId = orderParam.getOrgId();
        String time = orderParam.getTime();
        Integer number = orderParam.getNumber();
        Integer orderType = orderParam.getOrderType();
        QueryWrapper<PurchaseOrder> queryWrapper = new QueryWrapper<>();
        if (null != orderType) {
            queryWrapper.eq("order_type", orderType);
        }
        queryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId).orderByAsc("create_time");
        queryWrapper.select(" IFNULL(sum(CONVERT(purchase_money, DECIMAL(10,2))),'0') purchaseMoney,create_time createTime,IFNULL(count(DISTINCT order_number),'0') orderNumber");
        BigDecimal math = new BigDecimal("10000");
        List<PurchaseOrderMoneyVo> list = new ArrayList<>();

        if ( null == orderParam.getTimeType() || 0==(orderParam.getTimeType())) {
            Calendar calendar = Calendar.getInstance();
            if (null == number) {
                calendar.add(Calendar.YEAR, -3);
            } else {
                calendar.add(Calendar.YEAR, -number);
            }
            queryWrapper.between("create_time", calendar.getTime(), new Date());
            if ("".equals(orderParam.getOrderType()) || null == orderParam.getOrderType()) {
                queryWrapper.groupBy("DATE_FORMAT(create_time,'%Y')");
            } else {
                queryWrapper.groupBy("DATE_FORMAT(create_time,'%Y')");
            }

            List<PurchaseOrder> orders = purchaseOrderMapper.selectList(queryWrapper);
            if (orders.size() != 0) {
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy");
                for (PurchaseOrder order : orders) {
                    PurchaseOrderMoneyVo vo = new PurchaseOrderMoneyVo();
                    vo.setTime(sdf.format(order.getCreateTime()));
                    BigDecimal bd = new BigDecimal(order.getPurchaseMoney());
                    bd = bd.divide(math, 2, RoundingMode.HALF_UP);
                    vo.setOrderMoney(bd.toString());
                    vo.setOrderNumber(Integer.parseInt(order.getOrderNumber()));
                    list.add(vo);
                }
            }
        } else {
            Calendar calendar = Calendar.getInstance();
            if (null == number) {
                calendar.add(Calendar.MONTH, -13);
            } else {
                calendar.add(Calendar.MONTH, -number);
            }
            queryWrapper.between("create_time", calendar.getTime(), new Date());
            queryWrapper.groupBy("DATE_FORMAT(create_time,'%Y-%m')");
            if ("".equals(orderParam.getOrderType()) || null == orderParam.getOrderType()) {
                queryWrapper.groupBy("DATE_FORMAT(create_time,'%Y-%m')");
            } else {
                queryWrapper.groupBy("DATE_FORMAT(create_time,'%Y-%m')");
            }
            List<PurchaseOrder> orders = purchaseOrderMapper.selectList(queryWrapper);
            if (orders.size() != 0) {
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM");
                for (PurchaseOrder order : orders) {
                    PurchaseOrderMoneyVo vo = new PurchaseOrderMoneyVo();
                    vo.setTime(sdf.format(order.getCreateTime()));
                    BigDecimal bd = new BigDecimal(order.getPurchaseMoney());
                    bd = bd.divide(math, 2, RoundingMode.HALF_UP);
                    vo.setOrderMoney(bd.toString());
                    vo.setOrderNumber(Integer.parseInt(order.getOrderNumber()));
                    list.add(vo);
                }
            }
        }
        return list;
    }

    @Override
    public PurchaseOrderTypeVo orderType(PurchaseParam param) {
        String orgId = param.getOrgId();
        String maerName = param.getMaerName();
        String erpFnumber = param.getErpFnumber();
        String time = param.getTime();
        DateTime currentDate = new DateTime();
        if (StringUtils.isNotBlank(time)){
             currentDate = DateUtil.parse(time, "yyyy-MM");
        }

        DateTime beginOfMonth = DateUtil.beginOfMonth(currentDate);
        DateTime endOfMonth = DateUtil.endOfMonth(currentDate);
        DateTime beginOfYear = DateUtil.beginOfYear(currentDate);
        DateTime endOfYear = DateUtil.endOfYear(currentDate);
        DateTime lastCurrenDate  = DateUtil.offset(currentDate, DateField.YEAR,-1);
        DateTime lastbeginOfMonth = DateUtil.beginOfMonth(lastCurrenDate);
        DateTime lastendOfMonth = DateUtil.endOfMonth(lastCurrenDate);
        DateTime lastbeginOfYear = DateUtil.beginOfYear(lastCurrenDate);
        DateTime lastendOfYear = DateUtil.endOfYear(lastCurrenDate);

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        List<ErpBdRawMaterial> erpBdRawMaterials = erpBdRawMaterialMapper.selectList(new QueryWrapper<ErpBdRawMaterial>()
                .eq(StringUtils.isNotBlank(orgId), "org_id", orgId).eq(StringUtils.isNotBlank(erpFnumber), "erp_fnumber", erpFnumber));
        Integer materalId =0;
        if (ObjectUtil.isNotEmpty(erpBdRawMaterials)){
             materalId = erpBdRawMaterials.get(0).getMateralId();
        }
        PurchaseOrderTypeVo orderTypeVos = purchaseOrderMapper.listType(beginOfMonth,
                endOfMonth,
                beginOfYear,
                endOfYear,
                lastbeginOfMonth,
                lastendOfMonth,
                lastbeginOfYear,
                lastendOfYear,
               orgId,materalId);
        return orderTypeVos;
    }

    @Override
    public List<PurchaseMoneyVo> listMoney(PurchaseParam param) {
        String time = param.getTime();
        String substring = time.substring(0, 4);
        QueryWrapper<PurchaseOrder> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(StringUtils.isNotBlank(param.getOrgId()), "org_id", param.getOrgId());

        queryWrapper.between("create_time", Integer.parseInt(substring)+"-01-01 00:00:00",Integer.parseInt(substring) + "-12-31 23:59:59");
        queryWrapper.select(" IFNULL(sum(CONVERT(purchase_money, DECIMAL(10,2))),'0') purchaseMoney,create_time createTime");
        queryWrapper.groupBy("DATE_FORMAT(create_time,'%Y-%m')");
        List<PurchaseOrder> orders = purchaseOrderMapper.selectList(queryWrapper);
        queryWrapper.clear();
        queryWrapper.eq(StringUtils.isNotBlank(param.getOrgId()), "org_id", param.getOrgId());


        queryWrapper.between("create_time", Integer.parseInt(substring)-1 + "-01-01 00:00:00", Integer.parseInt(substring)-1 + "-12-31 23:59:59");
        queryWrapper.select(" IFNULL(sum(CONVERT(purchase_money, DECIMAL(10,2))),'0') purchaseMoney,create_time createTime");
        queryWrapper.groupBy("DATE_FORMAT(create_time,'%Y-%m')");
        List<PurchaseOrder> orders1 = purchaseOrderMapper.selectList(queryWrapper);
        List<PurchaseMoneyVo> list = new ArrayList<>();

        BigDecimal math = new BigDecimal("10000");
        for (int i = 1; i < 13; i++) {
            PurchaseMoneyVo vo = new PurchaseMoneyVo();
            vo.setTime(i + "月");
            for (PurchaseOrder order : orders) {
                Calendar cal = Calendar.getInstance();
                cal.setTime(order.getCreateTime());
                int month = cal.get(Calendar.MONTH) + 1; // 注意月份是从0开始计数的，所以需要加1
                if (i == month) {
                    BigDecimal bg = new BigDecimal(order.getPurchaseMoney());
                    bg = bg.divide(math, 2, RoundingMode.HALF_UP);
                    vo.setMoney(bg.toString());
                }
            }
            for (PurchaseOrder order : orders1) {
                Calendar cal = Calendar.getInstance();
                cal.setTime(order.getCreateTime());
                int month = cal.get(Calendar.MONTH) + 1; // 注意月份是从0开始计数的，所以需要加1
                if (i == month) {
                    BigDecimal bg = new BigDecimal(order.getPurchaseMoney());
                    bg = bg.divide(math, 2, RoundingMode.HALF_UP);
                    vo.setLastMoney(bg.toString());
                }
            }
            list.add(vo);
        }
        return list;
    }

    @Override
    public JSONArray materialType(PurchaseParam param) {
        QueryWrapper<PurchaseOrder> queryWrapper = new QueryWrapper<>();
        String time = param.getTime();
        DateTime dateTime = DateUtil.beginOfYear(new Date());
        if ("".equals(time) || null == time) {
            queryWrapper.ge("create_time", dateTime);
        } else {
            queryWrapper.between("create_time", time + "-01 00:00:00", time + "-31 23:59:59");
        }
        queryWrapper.eq(StringUtils.isNotBlank(param.getOrgId()), "org_id", param.getOrgId());
        BigDecimal math = new BigDecimal("10000");
        List<PurchaseMaterialVo> list = purchaseOrderMapper.getSupplier(queryWrapper);
        for (int i = 0; i < list.size(); i++) {
            BigDecimal bg = new BigDecimal(list.get(i).getValue());
//            bg = bg.divide(math, 2, RoundingMode.HALF_UP);
//            list.get(i).setValue(bg.toString());
            if (list.get(i).getName() == null) {
                list.remove(i);
            }
        }
        JSONArray jsonArray = new JSONArray(list);
        System.out.println("数组的大小" + jsonArray.size());
        JSONArray array = jsonArraySort(jsonArray, "value", true);
        return array;
    }

    @Override
    public  PurchaseAllParam getIndicator(DongShiPurchaseParam priceParam) {
        String orgId = priceParam.getOrgId();
        String time = priceParam.getTime();
        Date date ;
        if (StringUtils.isBlank(time)){
            date = DateUtil.beginOfMonth(new Date());
        }else {
            date = DateUtil.beginOfMonth(   DateUtil.parse(time,"yyyy-MM"));
        }
        DateTime lastYear = DateUtil.offsetMonth(date, -12);


        QueryWrapper<PurchaseOrder> wapper = new QueryWrapper<PurchaseOrder>();
        wapper.eq(StringUtils.isNotBlank(orgId),"org_id", orgId).between("create_time", date, DateUtil.endOfMonth(date));
        PurchaseAllParam purchaseAllParam =  purchaseOrderMapper.purchaseAll(wapper);
        wapper.clear();
        wapper.eq(StringUtils.isNotBlank(orgId),"org_id", orgId)
            .between("create_time", lastYear, DateUtil.endOfMonth(lastYear));
        PurchaseAllParam lasthaseAllParam =  purchaseOrderMapper.purchaseAll(wapper);

        //年统计
        wapper.clear();
        wapper.eq(StringUtils.isNotBlank(orgId),"org_id", orgId)
                .between("create_time", DateUtil.beginOfYear(date), DateUtil.endOfYear(date));
        PurchaseAllParam yaerHaseAllParam3 =  purchaseOrderMapper.purchaseAll(wapper);

        //年统计
        wapper.clear();
        wapper.eq(StringUtils.isNotBlank(orgId),"org_id", orgId)
                .between("create_time", DateUtil.beginOfYear(lastYear), DateUtil.endOfYear(lastYear));
        PurchaseAllParam lastYearhaseAllParam=  purchaseOrderMapper.purchaseAll(wapper);

    //计算同比
        purchaseAllParam.setCaiGouMoneyIsOn(getDouble(lasthaseAllParam.getCaiGouMoney()) <getDouble(purchaseAllParam.getCaiGouMoney()));
        purchaseAllParam.setKuCunMoneyIsOn(getDouble(lasthaseAllParam.getKuCunMoney()) <getDouble(purchaseAllParam.getKuCunMoney()));
        purchaseAllParam.setZhuYaoMoneyIsOn(getDouble(lasthaseAllParam.getZhuYaoMoney()) <getDouble(purchaseAllParam.getZhuYaoMoney()));
        purchaseAllParam.setZhuYaoNumIsOn(getDouble(lasthaseAllParam.getZhuYaoNum()) <getDouble(purchaseAllParam.getZhuYaoNum()));
        purchaseAllParam.setYvQiYingFuIsOn(getDouble(lasthaseAllParam.getYvQiYingFu()) <getDouble(purchaseAllParam.getYvQiYingFu()));
        purchaseAllParam.setZhuYaoYuanCaiLiaoMoneyIsOn(getDouble(lasthaseAllParam.getZhuYaoYuanCaiLiaoMoney()) <getDouble(purchaseAllParam.getZhuYaoYuanCaiLiaoMoney()));
        purchaseAllParam.setZhuYaoYuanCaiLiaoNumIsOn(getDouble(lasthaseAllParam.getZhuYaoYuanCaiLiaoNum()) <getDouble(purchaseAllParam.getZhuYaoYuanCaiLiaoNum()));

        purchaseAllParam.setCaiGouMoneyOnYear(calculateYOYChange(getDouble(purchaseAllParam.getCaiGouMoney()),getDouble(lasthaseAllParam.getCaiGouMoney())));
        purchaseAllParam.setKuCunMoneyOnYear(calculateYOYChange(getDouble(purchaseAllParam.getKuCunMoney()),getDouble(lasthaseAllParam.getKuCunMoney())));
        purchaseAllParam.setZhuYaoMoneyOnYear(calculateYOYChange(getDouble(purchaseAllParam.getZhuYaoMoney()),getDouble(lasthaseAllParam.getZhuYaoMoney())));
        purchaseAllParam.setZhuYaoNumOnYear(calculateYOYChange(getDouble(purchaseAllParam.getZhuYaoNum()),getDouble(lasthaseAllParam.getZhuYaoNum())));
        purchaseAllParam.setYvQiYingFuOnYear(calculateYOYChange(getDouble(purchaseAllParam.getYvQiYingFu()),getDouble(lasthaseAllParam.getYvQiYingFu())));
        purchaseAllParam.setZhuYaoYuanCaiLiaoMoneyOnYear(calculateYOYChange(getDouble(purchaseAllParam.getZhuYaoYuanCaiLiaoMoney()),getDouble(lasthaseAllParam.getZhuYaoYuanCaiLiaoMoney())));
        purchaseAllParam.setZhuYaoYuanCaiLiaoNumOnYear(calculateYOYChange(getDouble(purchaseAllParam.getZhuYaoYuanCaiLiaoNum()),getDouble(lasthaseAllParam.getZhuYaoYuanCaiLiaoNum())));

        purchaseAllParam.setCaiGouMoneyYear(yaerHaseAllParam3.getCaiGouMoney());
        purchaseAllParam.setZhuYaoMoneyYear(yaerHaseAllParam3.getZhuYaoMoney());
        purchaseAllParam.setZhuYaoNumYear(yaerHaseAllParam3.getZhuYaoNum());
        //计算同比
        purchaseAllParam.setYearCaiGouMoneyIsOn(getDouble(lastYearhaseAllParam.getCaiGouMoney()) <getDouble(yaerHaseAllParam3.getCaiGouMoney()));
        purchaseAllParam.setYearZhuYaoMoneyIsOn(getDouble(lastYearhaseAllParam.getZhuYaoMoney()) <getDouble(yaerHaseAllParam3.getZhuYaoMoney()));
        purchaseAllParam.setYearZhuYaoNumIsOn(getDouble(lastYearhaseAllParam.getZhuYaoNum()) <getDouble(yaerHaseAllParam3.getZhuYaoNum()));

        purchaseAllParam.setYearCaiGouMoneyOnYear(calculateYOYChange(getDouble(yaerHaseAllParam3.getCaiGouMoney()),getDouble(lastYearhaseAllParam.getCaiGouMoney())));
        purchaseAllParam.setYearZhuYaoMoneyOnYear(calculateYOYChange(getDouble(yaerHaseAllParam3.getZhuYaoMoney()),getDouble(lastYearhaseAllParam.getZhuYaoMoney())));
        purchaseAllParam.setYearZhuYaoNumOnYear(calculateYOYChange(getDouble(yaerHaseAllParam3.getZhuYaoNum()),getDouble(lastYearhaseAllParam.getZhuYaoNum())));


        return purchaseAllParam;
    }

    @Override
    public List<PurchaseAllParam> getCaiGouYvFuDetil(DongShiPurchaseParam priceParam) {
        String orgId = priceParam.getOrgId();
        Integer type = priceParam.getType();
        QueryWrapper<PurchaseOrder> wapper = new QueryWrapper<PurchaseOrder>();
        wapper.eq( StringUtils.isNotBlank(orgId),"org_id", orgId).orderByAsc("ymonth").groupBy("ymonth");
        List<PurchaseAllParam> busAllParams;
        if (ObjectUtil.isEmpty(type) || 0 ==  type){
             busAllParams= purchaseOrderMapper.getCaiGouYvFuDetailMonth(wapper);
        }else {
            busAllParams = purchaseOrderMapper.getCaiGouYvFuDetailYear(wapper);
        }
        return busAllParams;
    }

    @Override
    public List<PurchaseAllParam> getCaiGouYvFuQuShi(DongShiPurchaseParam priceParam) {
        String orgId = priceParam.getOrgId();
        Integer type = priceParam.getType();
        QueryWrapper<PurchaseOrder> wapper = new QueryWrapper<PurchaseOrder>();
        wapper.eq( StringUtils.isNotBlank(orgId),"org_id", orgId).orderByAsc("ymonth").groupBy("ymonth");
        List<PurchaseAllParam> busAllParams;
        if (ObjectUtil.isEmpty(type) || 0 ==  type){
            busAllParams= purchaseOrderMapper.getCaiGouYvFuDetailMonth(wapper);
        }else {
            busAllParams = purchaseOrderMapper.getCaiGouYvFuDetailYear(wapper);
        }

        for (int i = 0; i < busAllParams.size() -1 ; i++) {
            PurchaseAllParam  lastDsBus  = busAllParams.get(i);
            PurchaseAllParam dsBusAllParam = busAllParams.get(i + 1);
            dsBusAllParam.setCaiGouMoneyOnYear(calculateYOYZiJinYvE(getDouble(dsBusAllParam.getCaiGouMoney()),getDouble(lastDsBus.getCaiGouMoney()) ));
            dsBusAllParam.setCaiGouMoneyIsOn(getDouble(lastDsBus.getCaiGouMoney()) <getDouble(dsBusAllParam.getCaiGouMoney()));
        }
        if (ObjectUtil.isNotEmpty(busAllParams)){
            PurchaseAllParam dsBusAllParam = busAllParams.get(0);
            dsBusAllParam.setCaiGouMoneyIsOn(true);
            dsBusAllParam.setCaiGouMoneyOnYear("0%");
        }

        return busAllParams;
    }

    @Override
    public List<PurchaseYuanCaiLiaoParam>getZhuYaoYuanCaiLiao(DongShiPurchaseParam priceParam) {
        String orgId = priceParam.getOrgId();
        QueryWrapper<PurchasePrice> wapper = new QueryWrapper<PurchasePrice>();
//        Page<PurchaseYuanCaiLiaoParam> page = new Page<>(priceParam.getPageNum(), priceParam.getPageSize());
        wapper.eq( StringUtils.isNotBlank(orgId),"org_id", orgId).eq("material","羊毛粗脂").orderByAsc("ymonth").groupBy("ymonth");

        return  purchasePriceMapper.getZhuYaoYuanCaiLiao(wapper);
    }

    @Override
    public Page<GongYingShangParam> getGongYingShang(DongShiPurchaseParam priceParam) {
        String orgId = priceParam.getOrgId();
        QueryWrapper<PurchaseOrder> wapper = new QueryWrapper<PurchaseOrder>();
        Page<GongYingShangParam> page = new Page<>(priceParam.getPageNum(), priceParam.getPageSize());
        wapper.eq( StringUtils.isNotBlank(orgId),"org_id", orgId).groupBy("supplier");
        Page<GongYingShangParam> gongYingShang = purchasePriceMapper.getGongYingShang(page, wapper);
        if (ObjectUtil.isNotEmpty(gongYingShang.getRecords())){
            for (int i = 0; i < gongYingShang.getRecords().size(); i++) {
                GongYingShangParam gongYingShangParam = gongYingShang.getRecords().get(i);
                gongYingShangParam.setSort((priceParam.getPageNum()-1) *priceParam.getPageSize() + i+ 1 );
            }
        }

        return  gongYingShang;
    }

    @Override
    public List<String> getWuLiaoShaiXuan(DongShiPurchaseParam priceParam) {
        QueryWrapper<PurchaseOrder> wapper = new QueryWrapper<PurchaseOrder>();
        String orgId = priceParam.getOrgId();
        if ( StringUtils.isBlank(orgId)){
            wapper.eq("org_id", "1856160583875837954");
        }else {
            wapper.eq("org_id", orgId);
        }
        wapper.groupBy("fname");

        wapper.likeRight("erp_fnumber", "1002");

        return purchasePriceMapper.getWuLiaoShaiXuan(wapper);
    }

    @Override
    public List<DongShiKuCunWuliaoVoParam> getKuCunWuLiao(DongShiKuCunWuliaoParam priceParam) {
        QueryWrapper<PurchaseOrder> wapper = new QueryWrapper<PurchaseOrder>();
        String orgId = priceParam.getOrgId();
        String name = priceParam.getName();
        wapper.eq( StringUtils.isNotBlank(orgId),"org_id", orgId).eq(StringUtils.isNotBlank(name),"material_name",name);
        List<DongShiKuCunWuliaoVoParam> list = purchasePriceMapper.getKuCunWuLiao(wapper);
        return list;
    }


    @Override
    public List<ErpBdRawMaterial> getMater(PurchaseParam param) {
        String orgId = param.getOrgId();
        return purchasePriceMapper.getMater(orgId);
    }

    @Override
    public List<PurchaseAllParam> getKuChunYuanCaiFuDetil(DongShiPurchaseParam priceParam) {

        String orgId = priceParam.getOrgId();
        Integer type = priceParam.getType();
        QueryWrapper<PurchaseOrder> wapper = new QueryWrapper<PurchaseOrder>();
        wapper.eq( StringUtils.isNotBlank(orgId),"org_id", orgId).orderByAsc("ymonth").groupBy("ymonth");
        List<PurchaseAllParam> busAllParams;
        if (ObjectUtil.isEmpty(type) || 0 ==  type){
            busAllParams= purchaseOrderMapper.getKunChunJinE(wapper);
        }else {
            busAllParams = purchaseOrderMapper.getKunChunJinEYear(wapper);
        }
        return busAllParams;
    }

    @Override
    public List<PurchaseAllParam> getZhuYaoYuanCaiNumDetil(DongShiPurchaseParam priceParam) {

        String orgId = priceParam.getOrgId();
        Integer type = priceParam.getType();
        QueryWrapper<PurchaseOrder> wapper = new QueryWrapper<PurchaseOrder>();
        wapper.eq( StringUtils.isNotBlank(orgId),"org_id", orgId).orderByAsc("ymonth").groupBy("ymonth");
        wapper.eq("material_name", "羊毛粗脂");

        List<PurchaseAllParam> busAllParams;
        if (ObjectUtil.isEmpty(type) || 0 ==  type){
            busAllParams= purchaseOrderMapper.getZhuYaoYuanCaiNumDetil(wapper);
        }else {
            busAllParams = purchaseOrderMapper.getZhuYaoYuanCaiNumDetilYear(wapper);
        }
        return busAllParams;
    }

    private Double getDouble(String balance) {
        if (StringUtils.isBlank(balance)){
            return 0.0;
        }
        return Double.parseDouble(balance);
//        return  balance;
    }
    /*数组排序*/
    public static JSONArray jsonArraySort(JSONArray jsonArr, String sortKey, boolean is_desc) {
        //存放排序结果json数组
        JSONArray sortedJsonArray = new JSONArray();
        //用于排序的list
        List<JSONObject> jsonValues = new ArrayList<JSONObject>();
        //将参数json数组每一项取出，放入list
        if (jsonArr.size() < 6) {
            for (int i = 0; i < jsonArr.size(); i++) {
                jsonValues.add(jsonArr.getJSONObject(i));
            }
            //快速排序，重写compare方法，完成按指定字段比较，完成排序
            Collections.sort(jsonValues, new Comparator<JSONObject>() {
                //排序字段
                private final String KEY_NAME = sortKey;

                //重写compare方法
                @Override
                public int compare(JSONObject a, JSONObject b) {
                    //如果用String接会导致一位数和两位数混合比对的时候不能准确比出来，要用int类型接
                    //String valA = new String();
                    //String valB = new String();
                    Float valA = 0.00f;
                    Float valB = 0.00f;
                    try {
                        valA = Float.parseFloat(a.getString(KEY_NAME));
                        valB = Float.parseFloat(b.getString(KEY_NAME));
                    } catch (JSONException e) {
                        e.printStackTrace();
                    }
                    //是升序还是降序
                    if (is_desc) {
                        return -valA.compareTo(valB);
                    } else {
                        return -valB.compareTo(valA);
                    }

                }
            });
            //将排序后结果放入结果jsonArray
            for (int i = 0; i < jsonArr.size(); i++) {
                sortedJsonArray.add(jsonValues.get(i));
            }
        } else {
            for (int i = 0; i < 5; i++) {
                jsonValues.add(jsonArr.getJSONObject(i));
            }
            //快速排序，重写compare方法，完成按指定字段比较，完成排序
            Collections.sort(jsonValues, new Comparator<JSONObject>() {
                //排序字段
                private final String KEY_NAME = sortKey;

                //重写compare方法
                @Override
                public int compare(JSONObject a, JSONObject b) {
                    //如果用String接会导致一位数和两位数混合比对的时候不能准确比出来，要用int类型接
                    //String valA = new String();
                    //String valB = new String();
                    Float valA = 0.00f;
                    Float valB = 0.00f;
                    try {
                        valA = Float.parseFloat(a.getString(KEY_NAME));
                        valB = Float.parseFloat(b.getString(KEY_NAME));
                    } catch (JSONException e) {
                        e.printStackTrace();
                    }
                    //是升序还是降序
                    if (is_desc) {
                        return -valA.compareTo(valB);
                    } else {
                        return -valB.compareTo(valA);
                    }

                }
            });
            //将排序后结果放入结果jsonArray
            for (int i = 0; i < 5; i++) {
                sortedJsonArray.add(jsonValues.get(i));
            }
        }

        return sortedJsonArray;
    }

}
