package vip.xiaonuo.purchase.service.impl;

import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import vip.xiaonuo.erp.entity.ErpBdRawMaterial;
import vip.xiaonuo.erp.mapper.ErpBdRawMaterialMapper;
import vip.xiaonuo.purchase.entity.PurchasePrice;
import vip.xiaonuo.purchase.entity.vo.PurchasePriceVo;
import vip.xiaonuo.purchase.mapper.PurchasePriceMapper;
import vip.xiaonuo.purchase.param.PurchasePriceParam;
import vip.xiaonuo.purchase.service.IPurchasePriceService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 采购单品价格表 服务实现类
 * </p>
 *
 * @author author
 * @since 2024-11-27
 */
@Service
public class PurchasePriceServiceImpl extends ServiceImpl<PurchasePriceMapper, PurchasePrice> implements IPurchasePriceService {

    @Autowired
    private PurchasePriceMapper priceMapper;
    @Autowired
    private ErpBdRawMaterialMapper bdRawMaterialMapper;

    @Override
    public List<PurchasePriceVo> getList(PurchasePriceParam priceParam) {
        Integer number = priceParam.getNumber();

        QueryWrapper<PurchasePrice> queryWrapper = new QueryWrapper<>();

        Date date = new Date();


        Integer timeType = priceParam.getTimeType();
        if (null == timeType || 0 == timeType) {
            if (null == number) {
                number = 3;
            }
            List<PurchasePriceVo> monthlyPrices = new ArrayList<>() ;
            for (Integer integer = number; integer >= 0; integer--) {
                queryWrapper.clear();
                queryWrapper.eq(StringUtils.isNotBlank(priceParam.getOrgId()), "org_id", priceParam.getOrgId());
                queryWrapper.eq("fnumber", priceParam.getMaterial());
                Date  startDate = DateUtil.beginOfYear(date);
                Date  endDate = DateUtil.endOfYear(date);
                queryWrapper.between("create_time",DateUtil.offset(startDate, DateField.YEAR,-integer),DateUtil.offset(endDate, DateField.YEAR,-integer));
                PurchasePriceVo monthlyPrice = priceMapper.getMonthlyAvgPricesYear2(queryWrapper);
                queryWrapper.clear();
                queryWrapper.eq(StringUtils.isNotBlank(priceParam.getOrgId()), "org_id", priceParam.getOrgId());
                queryWrapper.eq("fnumber", priceParam.getMaterial());
                queryWrapper.between("create_time",DateUtil.offset(startDate, DateField.YEAR,-integer-1),DateUtil.offset(endDate, DateField.YEAR,-integer-1));
                PurchasePriceVo last = priceMapper.getMonthlyAvgPricesYear2(queryWrapper);

                if (ObjectUtil.isEmpty(monthlyPrice)){
                    monthlyPrice = new PurchasePriceVo();
                    monthlyPrice.setTime(DateUtil.format(DateUtil.offset(startDate, DateField.YEAR,-integer),"yyyy"));
                    monthlyPrice.setAvgPrice(0.0);
                }
                if (ObjectUtil.isEmpty(last)){
                    last = new PurchasePriceVo();
                    last.setAvgPrice(0.0);
                }
                monthlyPrice.setLastAvgPrice(String.format("%.2f",last.getAvgPrice()));

                monthlyPrice.setMonthOverMonthPct(calculateYOYZiJinYvE(monthlyPrice.getAvgPrice(),last.getAvgPrice()));
                monthlyPrice.setYearOverYearPct(calculateYOYZiJinYvE(monthlyPrice.getAvgPrice(),last.getAvgPrice()));
                monthlyPrices.add(monthlyPrice);
            }
            return  monthlyPrices ;

        } else {

            if (null == number) {
                number = 12;
            }
            List<PurchasePriceVo> monthlyPrices = new ArrayList<>();
            Date  startDate = DateUtil.beginOfMonth(date);
            Date  endDate = DateUtil.endOfMonth(date);
            for (Integer integer = number; integer >=0; integer--) {
                queryWrapper.clear();
                queryWrapper.eq(StringUtils.isNotBlank(priceParam.getOrgId()), "org_id", priceParam.getOrgId());
                queryWrapper.eq("fnumber", priceParam.getMaterial());
                queryWrapper.between("create_time",DateUtil.offset(startDate, DateField.MONTH,-integer),DateUtil.offset(endDate, DateField.MONTH,-integer));
                PurchasePriceVo monthlyPrice = priceMapper.getMonthlyAvgMonthYear2(queryWrapper);
                if (ObjectUtil.isEmpty(monthlyPrice)){
                    monthlyPrice = new PurchasePriceVo();
                    monthlyPrice.setTime(DateUtil.format(DateUtil.offset(startDate, DateField.MONTH,-integer),"yyyy-MM"));
                    monthlyPrice.setAvgPrice(0.0);
                }

                queryWrapper.clear();
                queryWrapper.eq(StringUtils.isNotBlank(priceParam.getOrgId()), "org_id", priceParam.getOrgId());
                queryWrapper.eq("fnumber", priceParam.getMaterial());
                queryWrapper.between("create_time",DateUtil.offset(startDate, DateField.MONTH,-integer-1),DateUtil.offset(endDate, DateField.MONTH,-integer-1));
                PurchasePriceVo lastMonth = priceMapper.getMonthlyAvgMonthYear2(queryWrapper);
                if (ObjectUtil.isEmpty(lastMonth)){
                    lastMonth = new PurchasePriceVo();
                    lastMonth.setTime(DateUtil.format(DateUtil.offset(startDate, DateField.MONTH,-integer),"yyyy-MM"));
                    lastMonth.setAvgPrice(0.0);
                }

                queryWrapper.clear();
                queryWrapper.eq(StringUtils.isNotBlank(priceParam.getOrgId()), "org_id", priceParam.getOrgId());
                queryWrapper.eq("fnumber", priceParam.getMaterial());
                queryWrapper.between("create_time",DateUtil.offset(startDate, DateField.MONTH,-integer-12),DateUtil.offset(endDate, DateField.MONTH,-integer-12));
                PurchasePriceVo lastYear = priceMapper.getMonthlyAvgMonthYear2(queryWrapper);
                if (ObjectUtil.isEmpty(lastYear)){
                    lastYear = new PurchasePriceVo();
                    lastYear.setTime(DateUtil.format(DateUtil.offset(startDate, DateField.MONTH,-integer),"yyyy-MM"));
                    lastYear.setAvgPrice(0.0);
                }
                monthlyPrice.setLastAvgPrice(String.format("%.2f",lastYear.getAvgPrice()));
                monthlyPrice.setMonthOverMonthPct(calculateYOYZiJinYvE(monthlyPrice.getAvgPrice(),lastMonth.getAvgPrice()));
                monthlyPrice.setYearOverYearPct(calculateYOYZiJinYvE(monthlyPrice.getAvgPrice(),lastMonth.getAvgPrice()));
                monthlyPrices.add(monthlyPrice);
            }
            return monthlyPrices;

//
//            Calendar calendar = Calendar.getInstance();
//            if (null == number) {
//                calendar.add(Calendar.MONTH, -13);
//            } else {
//                calendar.add(Calendar.MONTH, -number);
//            }
//            queryWrapper.between("create_time", calendar.getTime(), new Date());
//
//            List<PurchasePriceVo> monthlyPrices = priceMapper.getMonthlyAvgPrices(queryWrapper);
//            return calculatePriceChanges(monthlyPrices);

        }
//        return list;
    }


    public static String calculateYOYZiJinYvE(double currentValue, double previousValue) {
        // 处理零值情况
        if (previousValue == 0) {
            return "0";
        }
        // 计算同比变化率
        double percentageChange = ((currentValue - previousValue) /previousValue) * 100;
        if (percentageChange >500){
            return "100";
        }
        return String.format("%.2f", percentageChange);
    }

}
