package vip.xiaonuo.risk.service.impl;

import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import vip.xiaonuo.purchase.entity.PurchaseOrder;
import vip.xiaonuo.purchase.mapper.PurchaseOrderMapper;
import vip.xiaonuo.risk.entity.RiskContractWarning;
import vip.xiaonuo.risk.entity.RiskSupplier;
import vip.xiaonuo.risk.entity.RiskSupplierVo;
import vip.xiaonuo.risk.mapper.RiskContractWarningMapper;
import vip.xiaonuo.risk.mapper.RiskSupplierMapper;
import vip.xiaonuo.risk.param.RiskParam;
import vip.xiaonuo.risk.param.RiskSupplierParam;
import vip.xiaonuo.risk.service.IRiskSupplierService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * <p>
 * 风控供应商交货 服务实现类
 * </p>
 *
 * @author author
 * @since 2024-12-09
 */
@Service
public class RiskSupplierServiceImpl extends ServiceImpl<RiskSupplierMapper, RiskSupplier> implements IRiskSupplierService {

    @Autowired
    private RiskSupplierMapper supplierMapper;
    @Autowired
    private RiskContractWarningMapper riskContractWarningMapper;

    @Override
    public List<RiskSupplierVo> getList(RiskSupplierParam riskSupplierParam) {
        String orgId = riskSupplierParam.getOrgId();
        Integer timeType = riskSupplierParam.getTimeType();
        String supplier = riskSupplierParam.getSupplier();
        Integer number = riskSupplierParam.getNumber();
        QueryWrapper<RiskContractWarning> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        queryWrapper.eq(StringUtils.isNotBlank(supplier), "customer", supplier);
        List<RiskSupplierVo> list ;
        Date date = new Date();
        Date startdate ;
        if (1 == timeType) {
            if (null == number) {
                startdate =DateUtil.offsetMonth(DateUtil.beginOfMonth(date),-13);

            } else {
                startdate =DateUtil.offsetMonth(DateUtil.beginOfMonth(date),-number);
            }
            queryWrapper.ge("create_time", startdate);
            list = riskContractWarningMapper.riskSupplierParamMonth(queryWrapper);
        } else {
            if (null == number) {
                startdate =DateUtil.offset(DateUtil.beginOfYear(date), DateField.YEAR,-3);
            } else {
                startdate =DateUtil.offsetMonth(DateUtil.beginOfYear(date),-number);
            }

            queryWrapper.ge("create_time", startdate);
            list = riskContractWarningMapper.riskSupplierParamYear(queryWrapper);

        }

        return list;

    }

    @Override
    public JSONArray listSupplier(Map<String, String> paramsMap) {
        String orgId = paramsMap.get("orgId");
        QueryWrapper<RiskContractWarning> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        queryWrapper.groupBy("kehu");
        List<RiskContractWarning> orders = riskContractWarningMapper.selectList(queryWrapper);
        JSONArray jsonArray = new JSONArray();
        if (orders.size() != 0) {
            for (RiskContractWarning order : orders) {
                if (order.getKehu()!=null) {
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put("supplier", order.getKehu());
                    jsonObject.put("orgId", order.getOrgId());
                    jsonArray.add(jsonObject);
                }
            }
        }
        return jsonArray;
    }

}
