package vip.xiaonuo.sale.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import vip.xiaonuo.sale.entity.SaleDeliver;
import vip.xiaonuo.sale.entity.SaleDetail;
import vip.xiaonuo.sale.entity.SaleInvoice;
import vip.xiaonuo.sale.entity.vo.SaleInvoiceVo;
import vip.xiaonuo.sale.mapper.SaleDetailMapper;
import vip.xiaonuo.sale.mapper.SaleInvoiceMapper;
import vip.xiaonuo.sale.param.SaleInvoiceParam;
import vip.xiaonuo.sale.service.ISaleInvoiceService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 销售发票 服务实现类
 * </p>
 *
 * @author author
 * @since 2024-12-13
 */
@Service
public class SaleInvoiceServiceImpl extends ServiceImpl<SaleInvoiceMapper, SaleInvoice> implements ISaleInvoiceService {

    @Autowired
    private SaleInvoiceMapper invoiceMapper;
    @Autowired
    private SaleDetailMapper detailMapper;

    @Override
    public List<SaleInvoiceVo> getList(SaleInvoiceParam invoiceParam) {
        //改成合同金额
        String orgId = invoiceParam.getOrgId();
        String customer = invoiceParam.getCustomer();
        String materialName = invoiceParam.getMaterialName();
        Integer timeType = invoiceParam.getTimeType();
        Integer number = invoiceParam.getNumber();
        QueryWrapper<SaleDetail> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        queryWrapper.eq(StringUtils.isNotBlank(materialName), "produce_name", materialName);
        queryWrapper.eq(StringUtils.isNotBlank(customer), "customer", customer);
        queryWrapper.select(" count(*) saleNumber,create_time createTime,IFNULL(sum(sale_money),'0') saleMoney");
        if (0 == timeType) {
            Calendar calendar = Calendar.getInstance();
            if (null == number) {
                calendar.add(Calendar.YEAR, -2);
            } else {
                calendar.add(Calendar.YEAR, -number+1);
            }
            queryWrapper.between("create_time", calendar.getTime(), new Date());
            queryWrapper.groupBy("DATE_FORMAT(create_time,'%Y')");
        } else {
            Calendar calendar = Calendar.getInstance();
            if (null == number) {
                calendar.add(Calendar.MONTH, -12);
            } else {
                calendar.add(Calendar.MONTH, -number+1);
            }
            queryWrapper.between("create_time", calendar.getTime(), new Date());
            queryWrapper.groupBy("DATE_FORMAT(create_time,'%Y-%m')");
        }
        queryWrapper.orderByAsc("create_time");
        List<SaleInvoiceVo> list = new ArrayList<>();
        List<SaleDetail> saleInvoices = detailMapper.selectList(queryWrapper);
        BigDecimal math = new BigDecimal("10000");
        if (saleInvoices.size() != 0) {
            for (SaleDetail saleInvoice : saleInvoices) {
                SaleInvoiceVo vo = new SaleInvoiceVo();
                BigDecimal bg = new BigDecimal(saleInvoice.getSaleMoney());
                bg = bg.divide(math, 2, RoundingMode.HALF_UP);
                vo.setCreateTime(saleInvoice.getCreateTime());
                vo.setSaleInvoiceMoney(bg.toString());
                vo.setSaleInvoiceNumber(saleInvoice.getSaleNumber());
                list.add(vo);
            }
        }
        return list;
    }

}
