package vip.xiaonuo.sale.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.beans.factory.annotation.Autowired;
import vip.xiaonuo.money.entity.MoneyWriteOverdue;
import vip.xiaonuo.money.mapper.MoneyWriteOverdueMapper;
import vip.xiaonuo.production.entity.ProductionWorkstopPlan;
import vip.xiaonuo.sale.entity.*;
import vip.xiaonuo.sale.entity.vo.SaleDeliverVo;
import vip.xiaonuo.sale.entity.vo.SaleTargetDetailVo;
import vip.xiaonuo.sale.entity.vo.SaleTargetVo;
import vip.xiaonuo.sale.entity.vo.SaleWanChengLvVo;
import vip.xiaonuo.sale.mapper.*;
import vip.xiaonuo.sale.param.SaleCustomerParam;
import vip.xiaonuo.sale.param.SaleDetailPageParam;
import vip.xiaonuo.sale.param.SaleParam;
import vip.xiaonuo.sale.service.ISaleTargetService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 销售指标 服务实现类
 * </p>
 *
 * @author author
 * @since 2024-12-13
 */
@Service
public class SaleTargetServiceImpl extends ServiceImpl<SaleTargetMapper, SaleTarget> implements ISaleTargetService {

    @Autowired
    private SaleTargetMapper targetMapper;
    @Autowired
    private SaleInvoiceMapper invoiceMapper;
    @Autowired
    private SaleDeliverMapper deliverMapper;
    @Autowired
    private SaleOrderMapper orderMapper;
    @Autowired
    private SaleReceiveMapper receiveMapper;
    @Autowired
    private SaleDetailMapper saleDetailMapper;
    @Autowired
    private SaleOrderMapper saleOrderMapper;
    @Autowired
    private MoneyWriteOverdueMapper writeOverdueMapper;
    @Autowired
    private SaleVisitMapper saleVisitMapper;
    @Autowired
    private SalePlanMapper salePlanMapper;

    @Override
    public JSONArray getTarget(SaleParam saleParam) {
        String orgId = saleParam.getOrgId();
        String time = saleParam.getTime();
        DateTime parse = DateUtil.parse(time, "yyyy-MM");
        DateTime beginOfMonth = DateUtil.beginOfMonth(parse);
        DateTime endOfMonth = DateUtil.endOfMonth(parse);
        DateTime beginOfYear = DateUtil.beginOfYear(parse);
        DateTime endOfYear = DateUtil.endOfYear(parse);


        QueryWrapper<SaleTarget> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        queryWrapper.between("create_time", beginOfMonth, endOfMonth);
        SaleTarget monthTarget = saleVisitMapper.selectOneMonth(queryWrapper);

        queryWrapper.clear();
        queryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        queryWrapper.between("create_time", beginOfYear, endOfYear);
        SaleTarget yearTarget = saleVisitMapper.selectOneMonth(queryWrapper);

        //发货
        QueryWrapper<SaleDeliver> wrapper = new QueryWrapper<>();
        wrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        wrapper.between("create_time", beginOfMonth, endOfMonth);
        SaleDeliverVo monthDeliver =  deliverMapper.selectSumMoney(wrapper);
        monthTarget.setSaleDeliverMoney(monthDeliver.getSaleDeliverMoney());
        monthTarget.setSaleDeliverMoneyUs(monthDeliver.getUs());
        monthTarget.setSaleDeliverMoneyOs(monthDeliver.getOs());

        wrapper.clear();
        wrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        wrapper.between("create_time", beginOfYear, endOfYear);
        SaleDeliverVo yearDeliver =  deliverMapper.selectSumMoney(wrapper);
        yearTarget.setSaleDeliverMoney(yearDeliver.getSaleDeliverMoney());
        yearTarget.setSaleDeliverMoneyUs(yearDeliver.getUs());
        yearTarget.setSaleDeliverMoneyOs(yearDeliver.getOs());

        //订单
        QueryWrapper<SaleOrder> orderQueryWrapper = new QueryWrapper<>();
        orderQueryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        orderQueryWrapper.between("create_time", beginOfMonth, endOfMonth);
        SaleDeliverVo monthDeliverOrder =    orderMapper.selectSumMoney(orderQueryWrapper);
        monthTarget.setSaleOrderMoney(monthDeliverOrder.getSaleDeliverMoney());
        monthTarget.setSaleOrderMoneyUs(monthDeliverOrder.getUs());
        monthTarget.setSaleOrderMoneyOs(monthDeliverOrder.getOs());

        orderQueryWrapper.clear();
        orderQueryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        orderQueryWrapper.between("create_time", beginOfYear, endOfYear);
        SaleDeliverVo yearDeliverOrder =  orderMapper.selectSumMoney(orderQueryWrapper);
        yearTarget.setSaleOrderMoney(yearDeliverOrder.getSaleDeliverMoney());
        yearTarget.setSaleOrderMoneyUs(yearDeliverOrder.getUs());
        yearTarget.setSaleOrderMoneyOs(yearDeliverOrder.getOs());

        //合同
        QueryWrapper<SaleDetail> contrQueryWrapper = new QueryWrapper<>();
        contrQueryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        contrQueryWrapper.between("create_time", beginOfMonth, endOfMonth);
        SaleDeliverVo contDeliver =    saleDetailMapper.selectSumMoney(contrQueryWrapper);
        monthTarget.setContractMoney(contDeliver.getSaleDeliverMoney());
        monthTarget.setContractMoneyUs(contDeliver.getUs());
        monthTarget.setContractMoneyOs(contDeliver.getOs());

        contrQueryWrapper.clear();
        contrQueryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        contrQueryWrapper.between("create_time", beginOfYear, endOfYear);
        SaleDeliverVo contDeliverYear =    saleDetailMapper.selectSumMoney(contrQueryWrapper);
        yearTarget.setContractMoney(contDeliverYear.getSaleDeliverMoney());
        yearTarget.setContractMoneyUs(contDeliverYear.getUs());
        yearTarget.setContractMoneyOs(contDeliverYear.getOs());


        //已签未发金额
        QueryWrapper<SaleDetail> contrNoQueryWrapper = new QueryWrapper<>();
        contrNoQueryWrapper.eq(StringUtils.isNotBlank(orgId), "a.org_id", orgId);
        contrNoQueryWrapper.between("a.create_time", beginOfMonth, endOfMonth);
        contrNoQueryWrapper.isNull("c.sorr_derno");
        SaleDeliverVo contDeliverNoMonth =    saleDetailMapper.selectSumMoneyNo(contrNoQueryWrapper);
        monthTarget.setSaleInvoiceMoney(contDeliverNoMonth.getSaleDeliverMoney());
        monthTarget.setSaleInvoiceMoneyUs(contDeliverNoMonth.getUs());
        monthTarget.setSaleInvoiceMoneyOs(contDeliverNoMonth.getOs());

        contrNoQueryWrapper.clear();
        contrNoQueryWrapper.eq(StringUtils.isNotBlank(orgId), "a.org_id", orgId);
        contrNoQueryWrapper.between("a.create_time", beginOfYear, endOfYear);
        contrNoQueryWrapper.isNull("c.sorr_derno");
        SaleDeliverVo contDeliverNoYear =    saleDetailMapper.selectSumMoneyNo(contrNoQueryWrapper);
        yearTarget.setSaleInvoiceMoney(contDeliverNoYear.getSaleDeliverMoney());
        yearTarget.setSaleInvoiceMoneyUs(contDeliverNoYear.getUs());
        yearTarget.setSaleInvoiceMoneyOs(contDeliverNoYear.getOs());

        //计划
        QueryWrapper<SalePlan> salePlanQueryWrapper = new QueryWrapper<>();
        salePlanQueryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        salePlanQueryWrapper.between("create_time", beginOfMonth, endOfMonth);
        salePlanQueryWrapper.select(" IFNULL (SUM(sale_money) ,0)/10000   saleMoney");
        SalePlan monthPlan = salePlanMapper.selectOne(salePlanQueryWrapper);

        queryWrapper.clear();
        salePlanQueryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        salePlanQueryWrapper.between("create_time", beginOfYear, endOfYear);
        salePlanQueryWrapper.select(" IFNULL (SUM(sale_money) ,0)/10000  saleMoney");
        SalePlan yearPlan = salePlanMapper.selectOne(salePlanQueryWrapper);


        monthTarget.setCompleteRate(calculate(contDeliver.getSaleDeliverMoney(),monthPlan.getSaleMoney()));
        yearTarget.setCompleteRate(calculate(contDeliverYear.getSaleDeliverMoney(),yearPlan.getSaleMoney()));


        JSONArray jsonArray = new JSONArray();
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("type", "month");
        jsonObject.put("object", monthTarget);
        JSONObject jsonObject1 = new JSONObject();
        jsonObject1.put("type", "year");
        jsonObject1.put("object", yearTarget);
        jsonArray.add(jsonObject);
        jsonArray.add(jsonObject1);
        return jsonArray;
    }

    public  String calculate(String currentValue, String previousValue) {
        // 处理零值情况
        if (StringUtils.isBlank(previousValue)|| "0.00".equals(previousValue) ||"0".equals(previousValue) || "0.0".equals(previousValue)) {
            return "100";
        }
        // 计算同比变化率
        double percentageChange = (Double.parseDouble(currentValue)/Double.parseDouble( previousValue)) * 100;
        if (percentageChange >500){
            return "100";
        }
        return String.format("%.2f", percentageChange);
    }

    @Override
    public SaleTargetVo getTargetScreen(SaleParam saleParam) {
        String orgId = saleParam.getOrgId();
        String time = saleParam.getTime();
        String year = time.substring(0, 4);
        int lastYear = Integer.parseInt(year) - 1;
        String month = time.substring(5, 7);
        SaleTargetVo vo = new SaleTargetVo();


        //客户拜访次数

        QueryWrapper<SaleTarget> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        queryWrapper.between("create_time", time + "-01 00:00:00", time + "-31 23:59:59");
        queryWrapper.select(" IFNULL(sum(visit_time),'0') visitNumber");
        SaleTarget monthTarget = saleVisitMapper.selectOneMonth(queryWrapper);

        queryWrapper.clear();
        queryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        queryWrapper.between("create_time", year + "-01-01 00:00:00", year + "-12-31 23:59:59");
        queryWrapper.select(" IFNULL(sum(visit_time),'0') visitNumber");
        SaleTarget yearTarget = saleVisitMapper.selectOneMonth(queryWrapper);

        queryWrapper.clear();
        queryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        queryWrapper.between("create_time", lastYear + "-01-01 00:00:00", lastYear + "-12-31 23:59:59");
        queryWrapper.select(" IFNULL(sum(visit_time),'0') visitNumber");
        SaleTarget lastYearTarget = saleVisitMapper.selectOneMonth(queryWrapper);
        vo.setYearVisitNumRate(calculateYOYChange(Double.parseDouble(yearTarget.getVisitNumber()),Double.parseDouble(lastYearTarget.getVisitNumber())));

        queryWrapper.clear();
        queryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        queryWrapper.between("create_time", lastYear+"-"+month + "-01 00:00:00", lastYear+"-"+month +  "-31 23:59:59");
        queryWrapper.select(" IFNULL(sum(visit_time),'0') visitNumber");
        SaleTarget lastMonthTarget = saleVisitMapper.selectOneMonth(queryWrapper);


        //销售合同金额
        QueryWrapper<SaleDetail> orderQueryWrapper = new QueryWrapper<>();
        orderQueryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        orderQueryWrapper.between("create_time", time + "-01 00:00:00", time + "-31 23:59:59");
        orderQueryWrapper.select(" IFNULL(sum(CONVERT(sale_money, DECIMAL(10,2))),'0') saleMoney");
        SaleDetail monthOrder = saleDetailMapper.selectOne(orderQueryWrapper);
        orderQueryWrapper.clear();
        orderQueryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        orderQueryWrapper.between("create_time", lastYear+"-"+month + "-01 00:00:00", lastYear+"-"+month + "-31 23:59:59");
        orderQueryWrapper.select(" IFNULL(sum(CONVERT(sale_money, DECIMAL(10,2))),'0') saleMoney");
        SaleDetail yearOrder = saleDetailMapper.selectOne(orderQueryWrapper);

        orderQueryWrapper.clear();
        orderQueryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        orderQueryWrapper.between("create_time", year+"-01-01 00:00:00", year+ "-12-31 23:59:59");
        orderQueryWrapper.select(" IFNULL(sum(CONVERT(sale_money, DECIMAL(10,2))),'0') saleMoney");
        SaleDetail orderYear = saleDetailMapper.selectOne(orderQueryWrapper);
        orderQueryWrapper.clear();
        orderQueryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        orderQueryWrapper.between("create_time", lastYear+"-01-01 00:00:00", lastYear+ "-12-31 23:59:59");
        orderQueryWrapper.select(" IFNULL(sum(CONVERT(sale_money, DECIMAL(10,2))),'0') saleMoney");
        SaleDetail lastOrderYear = saleDetailMapper.selectOne(orderQueryWrapper);
        vo.setYearSaleMoneyRate(calculateYOYChange(Double.parseDouble(orderYear.getSaleMoney()),Double.parseDouble(lastOrderYear.getSaleMoney())));

        //销售发货金额
        QueryWrapper<SaleDeliver> deliverQueryWrapper = new QueryWrapper<>();
        deliverQueryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        deliverQueryWrapper.between("create_time", time + "-01 00:00:00", time + "-31 23:59:59");
        deliverQueryWrapper.select(" IFNULL(sum(CONVERT(sale_deliver_money, DECIMAL(10,2))),'0') saleDeliverMoney");
        SaleDeliver monthDeliver = deliverMapper.selectOne(deliverQueryWrapper);
        deliverQueryWrapper.clear();
        deliverQueryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        deliverQueryWrapper.between("create_time", lastYear+"-"+month + "-01 00:00:00", lastYear+"-"+month + "-31 23:59:59");
        deliverQueryWrapper.select(" IFNULL(sum(CONVERT(sale_deliver_money, DECIMAL(10,2))),'0') saleDeliverMoney");
        SaleDeliver yearDeliver = deliverMapper.selectOne(deliverQueryWrapper);

        deliverQueryWrapper.clear();
        deliverQueryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        deliverQueryWrapper.between("create_time", year+"-01-01 00:00:00", year+"-12-31 23:59:59");
        deliverQueryWrapper.select(" IFNULL(sum(CONVERT(sale_deliver_money, DECIMAL(10,2))),'0') saleDeliverMoney");
        SaleDeliver deliverYear = deliverMapper.selectOne(deliverQueryWrapper);

        deliverQueryWrapper.clear();
        deliverQueryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        deliverQueryWrapper.between("create_time", lastYear+"-01-01 00:00:00", lastYear+"-12-31 23:59:59");
        deliverQueryWrapper.select(" IFNULL(sum(CONVERT(sale_deliver_money, DECIMAL(10,2))),'0') saleDeliverMoney");
        SaleDeliver lastDeliverYear = deliverMapper.selectOne(deliverQueryWrapper);

        vo.setYearSaleDeliverMoneyRate(calculateYOYChange(Double.parseDouble(deliverYear.getSaleDeliverMoney()),Double.parseDouble(lastDeliverYear.getSaleDeliverMoney())));


        //逾期应收款金额
        QueryWrapper<MoneyWriteOverdue> saleReceiveQueryWrapper = new QueryWrapper<>();
        saleReceiveQueryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        saleReceiveQueryWrapper.eq("document_type", 1);
        saleReceiveQueryWrapper.between("deadline", time + "-01 00:00:00", time + "-31 23:59:59");
        saleReceiveQueryWrapper.select("IFNULL(sum(CONVERT(rmbje, DECIMAL(10,2))),'0') rmbje");
        MoneyWriteOverdue monthReceive = writeOverdueMapper.selectOne(saleReceiveQueryWrapper);
        saleReceiveQueryWrapper.clear();
        saleReceiveQueryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        saleReceiveQueryWrapper.between("deadline", lastYear+"-"+month + "-01 00:00:00", lastYear+"-"+month + "-31 23:59:59");
        saleReceiveQueryWrapper.select(" IFNULL(sum(CONVERT(rmbje, DECIMAL(10,2))),'0') rmbje");
        saleReceiveQueryWrapper.eq("document_type", 1);
        MoneyWriteOverdue yearReceive = writeOverdueMapper.selectOne(saleReceiveQueryWrapper);


        saleReceiveQueryWrapper.clear();
        saleReceiveQueryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        saleReceiveQueryWrapper.select("ROUND(IFNULL(sum(CONVERT(rmbje, DECIMAL(10,2))),0) /10000,2)  rmbje");
        saleReceiveQueryWrapper.eq("document_type", 1);
        MoneyWriteOverdue countReceive = writeOverdueMapper.selectOne(saleReceiveQueryWrapper);


        DecimalFormat def = new DecimalFormat("0.00");
        BigDecimal math = new BigDecimal("10000");

        if (lastMonthTarget.getVisitNumber().equals("0")) {
            vo.setVisitNumRate("0");
        } else {
            double v = (Double.valueOf(monthTarget.getVisitNumber()) - Double.valueOf(lastMonthTarget.getVisitNumber()))/Double.valueOf(lastMonthTarget.getVisitNumber());
            if (v >= 1.0) {
                vo.setVisitNumRate("100");
            } else {
                vo.setVisitNumRate(def.format(v * 100));
            }
        }
        vo.setVisitNum(monthTarget.getVisitNumber());
        vo.setVisitNumYear(yearTarget.getVisitNumber());

        QueryWrapper<SalePlan> salePlanQueryWrapper = new QueryWrapper<>();
        salePlanQueryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        salePlanQueryWrapper.between("create_time",time + "-01 00:00:00", time + "-31 23:59:59");
        salePlanQueryWrapper.select(" IFNULL (SUM(sale_money) ,0)   saleMoney");
        SalePlan monthPlan = salePlanMapper.selectOne(salePlanQueryWrapper);

        queryWrapper.clear();
        salePlanQueryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        salePlanQueryWrapper.between("create_time", year+"-01-01 00:00:00", year+ "-12-31 23:59:59");
        salePlanQueryWrapper.select(" IFNULL (SUM(sale_money) ,0)  saleMoney");
        SalePlan yearPlan = salePlanMapper.selectOne(salePlanQueryWrapper);


        vo.setComplete(calculate(monthOrder.getSaleMoney(),monthPlan.getSaleMoney()));
        vo.setCompleteRate("100");
        vo.setYearCompleteRate(calculate(orderYear.getSaleMoney(),yearPlan.getSaleMoney()));
        vo.setCompleteYear("100");
        if (yearOrder.getSaleMoney().equals("0")) {
            vo.setSaleMoneyRate("0");
        } else {
            double v = (Double.valueOf(monthOrder.getSaleMoney()) - Double.valueOf(yearOrder.getSaleMoney()))/Double.valueOf(yearOrder.getSaleMoney());
            vo.setSaleMoneyRate(def.format(v * 100));
        }
        BigDecimal bg = new BigDecimal(monthOrder.getSaleMoney());
        BigDecimal bg1 = new BigDecimal(orderYear.getSaleMoney());
        vo.setSaleMoney(bg.divide(math, 2, RoundingMode.HALF_UP).toString());
        vo.setSaleMoneyYear(bg1.divide(math, 2, RoundingMode.HALF_UP).toString());

        if (yearDeliver.getSaleDeliverMoney().equals("0")) {
            vo.setSaleDeliverMoneyRate("0");
        } else {
            double v = (Double.valueOf(monthDeliver.getSaleDeliverMoney()) - Double.valueOf(yearDeliver.getSaleDeliverMoney()))/Double.valueOf(yearDeliver.getSaleDeliverMoney());
            if (v >= 1.0) {
                vo.setSaleDeliverMoneyRate("100");
            } else {
                vo.setSaleDeliverMoneyRate(def.format(v * 100));
            }
        }
        BigDecimal bg2 = new BigDecimal(monthDeliver.getSaleDeliverMoney());
        vo.setSaleDeliverMoney(bg2.divide(math, 2, RoundingMode.HALF_UP).toString());

        BigDecimal bg4 = new BigDecimal(deliverYear.getSaleDeliverMoney());
        vo.setSaleDeliverMoneyYear(bg4.divide(math, 2, RoundingMode.HALF_UP).toString());

        if ("0".equals(yearReceive.getRmbje())) {
            vo.setReceiveMoneyRate("0");
        } else {
            double v = (Double.valueOf(monthReceive.getRmbje()) - Double.valueOf(yearReceive.getRmbje()))/Double.valueOf(yearReceive.getRmbje());
            if (v >= 1.0) {
                vo.setReceiveMoneyRate("100");
            } else {
                vo.setReceiveMoneyRate(def.format(v * 100));
            }
        }
        vo.setReceiveMoney(String.format("%.2f",Double.parseDouble(countReceive.getRmbje())));
        return vo;
    }



    public  String calculateYOYChange(double currentValue, double previousValue) {
        // 处理零值情况
        if (previousValue == 0) {
            return "0";
        }
        // 计算同比变化率
        double percentageChange = ((currentValue - previousValue) / previousValue) * 100;
        if (percentageChange >500){
            return "100";
        }
        return String.format("%.2f", percentageChange);
    }

    @Override
    public  List<SaleWanChengLvVo>  getCompleteDetail(SaleParam saleParam) {
        //TODO
        Integer type = saleParam.getType();
        String time = saleParam.getTime();
        String orgId = saleParam.getOrgId();
        QueryWrapper<SaleDetail> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
//        queryWrapper.between("create_time", time + "-01 00:00:00", time + "-31 23:59:59");
        List<SaleWanChengLvVo> saleDeliverVo ;
        if (type  == 0){
            queryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
            saleDeliverVo =  saleDetailMapper.selectSumMoneyMonth(queryWrapper);
            for (SaleWanChengLvVo saleWanChengLvVo : saleDeliverVo) {
                QueryWrapper<SalePlan> queryWrapperPlan = new QueryWrapper<>();
                queryWrapperPlan.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
                queryWrapperPlan.eq( "DATE_FORMAT(create_time,'%Y-%m') ", saleWanChengLvVo.getCreateTime());
                queryWrapperPlan.select( "ifnull(SUM(CONVERT(sale_money, DECIMAL(10,2))),0) saleMoney  ");
                SalePlan salePlan = salePlanMapper.selectOne(queryWrapperPlan);
                saleWanChengLvVo.setPlanMoney(salePlan.getSaleMoney());
                saleWanChengLvVo.setValue(calculate(saleWanChengLvVo.getSaleMoney(),salePlan.getSaleMoney()));
            }
        }else {
            queryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
            saleDeliverVo =  saleDetailMapper.selectSumMoneyYear(queryWrapper);
            for (SaleWanChengLvVo saleWanChengLvVo : saleDeliverVo) {
                QueryWrapper<SalePlan> queryWrapperPlan = new QueryWrapper<>();
                queryWrapperPlan.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
                queryWrapperPlan.eq( "DATE_FORMAT(create_time,'%Y') ", saleWanChengLvVo.getCreateTime());
                queryWrapperPlan.select( "ifnull(SUM(CONVERT(sale_money, DECIMAL(10,2))),0) saleMoney  ");
                SalePlan salePlan = salePlanMapper.selectOne(queryWrapperPlan);
                saleWanChengLvVo.setPlanMoney(salePlan.getSaleMoney());
                saleWanChengLvVo.setValue(calculate(saleWanChengLvVo.getSaleMoney(),salePlan.getSaleMoney()));
            }
        }

        return saleDeliverVo;
    }

    @Override
    public List<SaleTargetDetailVo> getVisitNumDetail(SaleParam saleParam) {
        Integer type = saleParam.getType();
        String time = saleParam.getTime();
        String orgId = saleParam.getOrgId();
        QueryWrapper<SaleVisit> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
//        queryWrapper.between("create_time", time + "-01 00:00:00", time + "-31 23:59:59");
        queryWrapper.orderByAsc("create_time");
        if (0 == type) {
            queryWrapper.select("IFNULL(sum(visit_time),0) visitTime,create_time createTime");
            queryWrapper.groupBy("DATE_FORMAT(create_time,'%y-%m')");
        } else {
            queryWrapper.select("IFNULL(sum(visit_time),0) visitTime,create_time createTime");
            queryWrapper.groupBy("DATE_FORMAT(create_time,'%y')");
        }
//        saleVisitMapper.selectOneMonth(queryWrapper)

        List<SaleVisit> targets = saleVisitMapper.selectList(queryWrapper);
        List<SaleTargetDetailVo> list = new ArrayList<>();
        if (targets.size() != 0) {
            for (SaleVisit target : targets) {
                SaleTargetDetailVo vo = new SaleTargetDetailVo();
                if (0 == type) {
                    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM");
                    vo.setCreateTime(sdf.format(target.getCreateTime()));
                } else {
                    SimpleDateFormat sdf = new SimpleDateFormat("yyyy");
                    vo.setCreateTime(sdf.format(target.getCreateTime()));
                }
                vo.setValue(target.getVisitTime()+"");
                list.add(vo);
            }
        }
        return list;
    }

    @Override
    public List<SaleTargetDetailVo> getSaleMoneyDetail(SaleParam saleParam) {
        Integer type = saleParam.getType();
        String time = saleParam.getTime();
        String orgId = saleParam.getOrgId();
        QueryWrapper<SaleDetail> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
//        queryWrapper.between("create_time", time + "-01 00:00:00", time + "-31 23:59:59");
        queryWrapper.orderByAsc("create_time");
        if (0 == type) {
            queryWrapper.select("round (IFNULL(sum(CONVERT(sale_money, DECIMAL(10,2))),0)/10000 ,2) saleMoney,create_time createTime");
            queryWrapper.groupBy("DATE_FORMAT(create_time,'%y-%m')");
        } else {
            queryWrapper.select("round (IFNULL(sum(CONVERT(sale_money, DECIMAL(10,2))),0) /10000 ,2) saleMoney,create_time createTime");
            queryWrapper.groupBy("DATE_FORMAT(create_time,'%y')");
        }
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM");
        SimpleDateFormat sdfy = new SimpleDateFormat("yyyy");
        List<SaleDetail> targets = saleDetailMapper.selectList(queryWrapper);
        List<SaleTargetDetailVo> list = new ArrayList<>();
        if (targets.size() != 0) {
            for (SaleDetail target : targets) {
                SaleTargetDetailVo vo = new SaleTargetDetailVo();
                if (0 == type) {
                    vo.setCreateTime(sdf.format(target.getCreateTime()));
                } else {
                    vo.setCreateTime(sdfy.format(target.getCreateTime()));
                }
                vo.setValue(target.getSaleMoney());
                list.add(vo);
            }
        }
        return list;
    }

    @Override
    public Page<SaleVisit> listTop(SaleDetailPageParam customerParam) {
        Integer pageNum = customerParam.getPageNum();
        Integer pageSize = customerParam.getPageSize();
        String orgId = customerParam.getOrgId();
        Page<SaleVisit> page = new Page<>(pageNum, pageSize);
        QueryWrapper<SaleVisit> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);
        queryWrapper.select(" IFNULL(sum(visit_time),'0') visitNumber,kehu customerName");
        queryWrapper.orderByDesc("visitNumber");
        queryWrapper.groupBy("kehu");
        Page<SaleVisit> targets = saleVisitMapper.selectPage(page, queryWrapper);
        List<SaleVisit> records = targets.getRecords();
        if (ObjectUtil.isNotEmpty(records)){
            for (int i = 0; i < records.size(); i++) {
                SaleVisit workstopPlan = records.get(i);
                workstopPlan.setId((pageNum-1) *pageSize + i+ 1 );
            }
        }

        return targets;
    }

    @Override
    public List<SaleTargetDetailVo> getSaleFaHuoDetail(SaleParam saleParam) {
        Integer type = saleParam.getType();
        String orgId = saleParam.getOrgId();
        QueryWrapper<SaleDeliver> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(StringUtils.isNotBlank(orgId), "org_id", orgId);

        queryWrapper.orderByAsc("create_time");
        if (0 == type) {
            queryWrapper.select("round(IFNULL(sum(CONVERT(sale_deliver_money, DECIMAL(10,2))),'0') /10000,2) saleDeliverMoney,create_time createTime");
            queryWrapper.groupBy("DATE_FORMAT(create_time,'%y-%m')");
        } else {
            queryWrapper.select("round(IFNULL(sum(CONVERT(sale_deliver_money, DECIMAL(10,2))),'0') / 10000 ,2)saleDeliverMoney,create_time createTime");
            queryWrapper.groupBy("DATE_FORMAT(create_time,'%y')");
        }

        List<SaleDeliver> targets = deliverMapper.selectList(queryWrapper);
        List<SaleTargetDetailVo> list = new ArrayList<>();
        BigDecimal math = new BigDecimal("10000");
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM");
        SimpleDateFormat sdfy = new SimpleDateFormat("yyyy");
        if (targets.size() != 0) {
            for (SaleDeliver target : targets) {
                SaleTargetDetailVo vo = new SaleTargetDetailVo();
                if (0 == type) {
                    vo.setCreateTime(sdf.format(target.getCreateTime()));
                } else {
                    vo.setCreateTime(sdfy.format(target.getCreateTime()));
                }
                vo.setValue(target.getSaleDeliverMoney());
                list.add(vo);
            }
        }
        return list;
    }

}
